/* BASIC/LL compiler by YT */

/* In future, this compiler be bootstrap.
 * So, C version is written with BASIC future only now. */

#include "libbasicll.h"

/* usage */

STRING_CONST(version, "0.05");

void usage(void)
{
	print_string(0, S("BASIC/LL version ").data);
	print_string(0, version.data);
	print_line(0);
	print_string(0, S("Usage: basicll [--version][--help][-r][-x][-c][-S][-o <file>] <file>.bas").data);
	print_line(0);
}

/* platform dependency */

#ifdef __WIN32__
	STRING_CONST(dir_sep, "\\");
#else
	STRING_CONST(dir_sep, "/");
#endif

/* types */

enum type_kind_t { tk_ordinal, tk_float, tk_array };

struct type_t {
	struct type_t const *next;
	STRING name;
	CHARACTER postfix;
	INTEGER size;
	enum type_kind_t kind;
	INTEGER lbound;
	struct type_t const *element;
};

struct type_t const logical_type = {NULL, S("LOGICAL").data, '\0', sizeof(LOGICAL), tk_ordinal, 0, NULL};
struct type_t const character_type = {&logical_type, S("CHARACTER").data, sizeof(CHARACTER), '\0', tk_ordinal, 0, NULL};
struct type_t const string_type = {&character_type, S("STRING").data, '$', sizeof(void *), tk_array, 1, &character_type};
struct type_t const integer_type = {&string_type, S("INTEGER").data, '%', sizeof(INTEGER), tk_ordinal, 0, NULL};
struct type_t const single_type = {&integer_type, S("SINGLE").data, '!', sizeof(SINGLE), tk_float, 0, NULL};
struct type_t const double_type = {&single_type, S("DOUBLE").data, '#', sizeof(DOUBLE), tk_float, 0, NULL};

struct type_t const *get_type_array_of(struct type_t const **list, int lbound, struct type_t const *element)
{
	struct type_t const *i;
	struct type_t *new_type;
	for(i = *list; i != NULL; i = i->next){
		if(i->element == element && i->lbound == lbound){
			return i;
		}
	}
	new_type = allocate(sizeof(struct type_t));
	new_type->next = *list;
	new_type->name = NULL;
	new_type->postfix = '\0';
	new_type->size = sizeof(void *);
	new_type->kind = tk_array;
	new_type->lbound = lbound;
	new_type->element = element;
	*list = new_type;
	return new_type;
}

void free_type_list(struct type_t const *list)
{
	while(list != NULL && list != &double_type){
		struct type_t const *next = list->next;
		string_release(list->name);
		deallocate((void *)list);
		list = next;
	}
}

/* parts of compiler */

STRING_CONST(main_template,
/* #ifdef BASIC_CONTROL_STACK */
"declare i32* @enter_control(i32, i32*)\r"
"declare void @leave_control(i32, i32*)\r"
"declare void @back_control(i32, i32*) noreturn\r"
/* #endif */
"declare void @END() noreturn\r"
"declare i32 @ERR()\r"
"declare i8* @allocate(i32)\r"
"declare void @string_addref(i8*)\r"
"declare void @string_release(i8*)\r"
"declare i8* @string_concatenate(i8*, i8*)\r"
"declare i32 @string_compare(i8*, i8*)\r"
"declare i32 @power_integer(i32, i32)\r"
"declare float @power_float(float, float)\r"
"declare double @power_double(double, double)\r"
"declare void @read_string(i8***, i8**)\r"
"declare void @read_double(i8***, double*)\r"
"declare void @OPEN(i8*, i32, i32)\r"
"declare void @CLOSE(i32)\r"
"declare void @print_character(i32, i8)\r"
"declare void @print_string(i32, i8*)\r"
"declare void @print_double(i32, double)\r"
"declare void @print_line(i32)\r"
"declare void @print_using_string(i32, i8*, i32*, i8*)\r"
"declare void @print_using_double(i32, i8*, i32*, double)\r"
"declare void @print_using_rest(i32, i8*, i32*)\r"
"declare void @write_string(i32, i8*)\r"
"declare void @KILL(i8*)\r"
"declare void @program(i32)\r"
"declare i32 @_setjmp(i32*)\r"
"@basic_argc = external global i32\r"
"@basic_args = external global i8**\r"
"@basic_on_exit = external global [16 x i32]\r"
"@basic_current_line = external global i32\r"
"@basic_exit_code = external global i32\r"
"%error_message_t = type {i32, i32, [7 x i8]}\r"
"@error_message = internal constant %error_message_t {i32 -1, i32 6, [7 x i8] c\"Error!\\00\"}\r"
"define i32 @main(i32 %argc, i8** %args) {\r"
"\tstore i32 %argc, i32* @basic_argc\r"
"\tstore i8** %args, i8*** @basic_args\r"
"\t%tmp3 = call i32 @_setjmp(i32* getelementptr ([16 x i32]* @basic_on_exit, i32 0, i32 0))\r"
"\t%tmp4 = icmp eq i32 %tmp3, 0\r"
"\tbr i1 %tmp4, label %execute, label %error\r"
"execute:\r"
"\tcall void @program(i32 0)\r"
"\tbr label %exit\r"
"exit:\r"
"\t%tmp6 = load i32* @basic_exit_code\r"
"\tret i32 %tmp6\r"
"error:\r"
"\t%err = call i32 @ERR()\r"
"\t%tmp8 = icmp eq i32 %err, 0\r"
"\tbr i1 %tmp8, label %exit, label %report\r"
"report:\r"
"\tcall void @print_string(i32 -2, i8* getelementptr (%error_message_t* @error_message, i32 0, i32 2, i32 0))\r"
"\t%tmp1011 = sitofp i32 %err to double\r"
"\tcall void @print_double(i32 -2, double %tmp1011)\r"
"\tcall void @print_line(i32 -2)\r"
"\t%exerr = add i32 %err, 100\r"
"\tret i32 %exerr\r"
"}\r");

void compile_time_error(INTEGER line_number, STRING message) __attribute__((noreturn));
void compile_time_error(INTEGER line_number, STRING message)
{
	STRING_CONST(separator, ": ");
	print_double(standard_error, line_number);
	print_string(standard_error, separator.data);
	print_string(standard_error, message);
	print_line(standard_error);
	ERROR(syntax_error);
}

void skip_spaces(STRING s, INTEGER *index)
{
	while(s[*index] == ' ' || s[*index] == '\t'){
		++ *index;
	}
	if(s[*index] == '\''){
		*index = LEN(s);
	}
}

INTEGER is_word1(CHARACTER c)
{
	return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z');
}

INTEGER is_word2(CHARACTER c)
{
	return is_word1(c) || (c >= '0' && c <= '9') || (c == '_');
}

INTEGER take_word(STRING line, INTEGER index)
{
	INTEGER result = index;
	if(is_word1(line[result])){
		++ result;
		while(is_word2(line[result])){
			++ result;
		}
	}
	return result;
}

INTEGER take_line_number(STRING line, INTEGER index)
{
	INTEGER result = index;
	while(line[result] >= '0' && line[result] <= '9'){
		++ result;
	}
	return result;
}

STRING take_string_literal(STRING line, INTEGER *index)
{
	INTEGER start;
	STRING result;
	++ *index;
	start = *index;
	while(line[*index] != '\"' && line[*index] != '\0'){
		++ *index;
	}
	result = MID(line, start + 1, *index - start);
	if(line[*index] == '\"'){
		++ *index;
	}
	return result;
}

STRING to_internal_name(STRING name)
{
	STRING_CONST(prefix, "@v");
	STRING result = prefix.data;
	CHARACTER c;
	INTEGER i;
	string_addref(result);
	c = name[LEN(name) - 1];
	if(is_word2(c)){
		string_append(&result, name);
	}else{
		STRING temp = name;
		string_addref(temp);
		if(c == ')'){
			set_length(&temp, LEN(name) - 3);
			string_append(&result, temp);
			set_length(&result, LEN(result) + 3);
			i = LEN(result) - 3;
			((char *)result)[i + 2] = 'a';
			c = name[LEN(name) - 3];
		}else{
			set_length(&temp, LEN(name) - 1);
			string_append(&result, temp);
			set_length(&result, LEN(result) + 2);
			i = LEN(result) - 2;
		}
		((char *)result)[i] = '$';
		((char *)result)[i + 1] = (c == '%') ? 'i' : (c == '!') ? 'f' : (c == '#') ? 'd' : (c == '$') ? 's' : 'm';
		string_release(temp);
	}
	return result;
}

enum identifier_state_t { unused, referenced, defined };

struct label_t {
	struct label_t *next;
	STRING label;
	INTEGER number;
	INTEGER data_position;
	enum identifier_state_t state;
	INTEGER line_number_in_file;
};

struct label_t *get_label(struct label_t **list, STRING label, INTEGER number)
{
	struct label_t *result;
	struct label_t *i;
	INTEGER min = 0;
	for(i = *list; i != NULL; i = i->next){
		if(string_compare(i->label, label) == 0){
			return i;
		}
		if(i->number < min){
			min = i->number;
		}
	}
	result = allocate(sizeof(struct label_t));
	result->next = *list;
	result->label = label;
	string_addref(result->label);
	result->number = (number != 0) ? number : min - 1;
	result->data_position = -1;
	result->state = unused;
	*list = result;
	return result;
}

void free_label_list(struct label_t *list)
{
	while(list != NULL){
		struct label_t *next = list->next;
		string_release(list->label);
		deallocate(list);
		list = next;
	}
}

struct user_defined_t {
	struct user_defined_t *next;
	STRING name;
	struct type_t const *type;
} user_defined_t;

struct user_defined_t *query_user_defined(struct user_defined_t *list, STRING name)
{
	struct user_defined_t *i;
	struct user_defined_t *result = NULL;
	for(i = list; i != NULL; i = i->next){
		if(string_compare(i->name, name) == 0){
			result = i;
			break;
		}
	}
	return result;
}

struct user_defined_t *get_user_defined(struct user_defined_t **list, STRING name, struct type_t const *type)
{
	struct user_defined_t *result = query_user_defined(*list, name);
	if(result == NULL || (result->type != type)){
		STRING n2 = name;
		string_addref(n2);
		result = NULL;
		if(type->postfix != '\0'){
			set_length(&n2, LEN(name) + 1);
			((char *)n2)[LEN(name)] = type->postfix;
			result = query_user_defined(*list, n2);
		}else if(type->kind == tk_array){
			set_length(&n2, LEN(name) + 3);
			((char *)n2)[LEN(name)] = type->element->postfix;
			((char *)n2)[LEN(name) + 1] = '(';
			((char *)n2)[LEN(name) + 2] = ')';
			result = query_user_defined(*list, n2);
		}
		if(result == NULL){
			result = allocate(sizeof(struct user_defined_t));
			result->next = *list;
			result->name = n2;
			string_addref(result->name);
			result->type = type;
			*list = result;
		}
		string_release(n2);
	}
	return result;
}

void free_user_defined_list(struct user_defined_t *list)
{
	while(list != NULL){
		struct user_defined_t *next = list->next;
		string_release(list->name);
		deallocate(list);
		list = next;
	}
}

struct string_literal_t {
	struct string_literal_t *next;
	INTEGER number;
	STRING value;
} string_literal_t;

void free_string_literal_list(struct string_literal_t *list)
{
	while(list != NULL){
		struct string_literal_t *next = list->next;
		string_release(list->value);
		deallocate(list);
		list = next;
	}
}

struct temp_in_expression_t {
	struct temp_in_expression_t *next;
	INTEGER temp;
	struct type_t const *type;
};

void free_temp_list(struct temp_in_expression_t *list)
{
	while(list != NULL){
		struct temp_in_expression_t *next = list->next;
		deallocate(list);
		list = next;
	}
}

struct nest_t {
	struct nest_t *next;
	enum basic_control_kind_t kind;
	struct user_defined_t *control;
	INTEGER label;
	INTEGER level;
};

void free_nest_list(struct nest_t *list)
{
	while(list != NULL){
		struct nest_t *next = list->next;
		list = next;
	}
}

struct context_t {
	STRING code;
	struct label_t *label_list;
	struct user_defined_t *user_defined_list;
	struct string_literal_t *string_literal_list;
	struct temp_in_expression_t *temp_list;
	struct nest_t *nest_list;
	INTEGER line_number_in_file;
	INTEGER temp;
	INTEGER level;
	STRING *data;
	INTEGER data_count;
	INTEGER data_used;
	INTEGER list_used;
	INTEGER jump_table_used;
	struct type_t const *type_list;
	struct type_t const *default_type['Z' - 'A' + 1];
};

LOGICAL next_word_is(STRING line, INTEGER index, struct context_t *context, STRING word)
{
	LOGICAL result;
	INTEGER end = take_word(line, index);
	if(end - index != LEN(word)){
		result = 0;
	}else{
		STRING m = MID(line, index + 1, end - index);
		STRING u = UCASE(m);
		result = (string_compare(u, word) == 0
			&& query_user_defined(context->user_defined_list, word) == NULL);
		string_release(m);
		string_release(u);
	}
	return result;
}

struct expression_t {
	INTEGER result;
	struct type_t const *type;
};

/* forward */

void to_integer(struct context_t *context, struct expression_t *expression, INTEGER *result);
void to_single(struct context_t *context, struct expression_t *expression, INTEGER *result);
void to_double(struct context_t *context, struct expression_t *expression, INTEGER *result);
void load_variable(struct context_t *context, struct user_defined_t *variable, struct expression_t *expression);
void store_variable(struct context_t *context, struct user_defined_t *variable, struct expression_t *expression);
void parse_type_postfix(STRING line, INTEGER *index, struct context_t *context, STRING name, struct type_t const **type);
LOGICAL parse_expression(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression, LOGICAL error);
void parse_logical_expression(STRING line, INTEGER *index, struct context_t *context, INTEGER *result);
void parse_integer_expression(STRING line, INTEGER *index, struct context_t *context, INTEGER *result);
void parse_single_expression(STRING line, INTEGER *index, struct context_t *context, INTEGER *result);
void parse_double_expression(STRING line, INTEGER *index, struct context_t *context, INTEGER *result);
void parse_string_expression(STRING line, INTEGER *index, struct context_t *context, INTEGER *result);
void parse_symbol(STRING line, INTEGER *index, struct context_t *context, CHARACTER c);
LOGICAL parse_statement(STRING line, INTEGER *index, struct context_t *context, struct user_defined_t **assignment);
LOGICAL parse_multi_statement(STRING line, INTEGER *index, struct context_t *context, LOGICAL multi_statement);
void cleanup(struct context_t *context);

/* commands */

STRING_CONST(r_close, "CLOSE");

LOGICAL s_close(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(template, "\tcall void @CLOSE(i32 %tmp#)\r");

	INTEGER using;
	INTEGER file;
	*index += LEN(r_close.data);
	skip_spaces(line, index);
	parse_symbol(line, index, context, '#');
	parse_integer_expression(line, index, context, &file);
	using = 0;
	using_double(&context->code, template.data, &using, file);
	using_rest(&context->code, template.data, &using);
	cleanup(context);
	return 1;
}

STRING_CONST(r_data, "DATA");

LOGICAL s_data(STRING line, INTEGER *index, struct context_t *context)
{
	*index += LEN(r_data.data);
	skip_spaces(line, index);
	while(line[*index] != ':' && line[*index] != '\'' && line[*index] != '\0'){
		STRING s = NULL;
		if(line[*index] == ','){
			++ *index;
		}if(line[*index] == '"'){
			s = take_string_literal(line, index);
		}else{
			INTEGER start = *index;
			while(line[*index] != ',' && line[*index] != ':' && line[*index] != '\'' && line[*index] != '\0'){
				++ *index;
			}
			s = MID(line, start + 1, *index - start);
			if(line[*index] == ','){
				++ *index;
			}
		}
		++ context->data_count;
		context->data = reallocate(context->data, context->data_count * sizeof(STRING));
		context->data[context->data_count - 1] = s;
		string_addref(context->data[context->data_count - 1]);
		context->data_used = 1;
		skip_spaces(line, index);
		string_release(s);
	}
	return 1;
}

STRING_CONST(r_defdbl, "DEFDBL");
STRING_CONST(r_defint, "DEFINT");
STRING_CONST(r_defsng, "DEFSNG");
STRING_CONST(r_defstr, "DEFSTR");

LOGICAL s_deftype(STRING line, INTEGER *index, struct context_t *context)
{
	LOGICAL result = 0;
	CHARACTER s;
	CHARACTER e;
	INTEGER i;
	struct type_t const *type;
	if(next_word_is(line, *index, context, r_defdbl.data)){
		type = &double_type;
	}else if(next_word_is(line, *index, context, r_defint.data)){
		type = &integer_type;
	}else if(next_word_is(line, *index, context, r_defsng.data)){
		type = &single_type;
	}else{
		type = &string_type;
	}
	*index += 6;
	skip_spaces(line, index);
	while(is_word1(line[*index])){
		result = 1;
		s = line[*index];
		++ *index;
		skip_spaces(line, index);
		if(line[*index] == '-'){
			++ *index;
			skip_spaces(line, index);
			if(! is_word1(line[*index])){
				STRING_CONST(message, "Invalid initial character.");
				compile_time_error(context->line_number_in_file, message.data);
			}
			e = line[*index];
			if(s >= e){
				STRING_CONST(message, "Invalid initial character range.");
				compile_time_error(context->line_number_in_file, message.data);
			}
		}else{
			e = s;
		}
		for(i = s; i <= e; ++ i){
			context->default_type[i - 'A'] = type;
		}
		if(line[*index] != ','){
			break;
		}
		++ *index;
		skip_spaces(line, index);
	}
	return result;
}

STRING_CONST(r_dim, "DIM");

LOGICAL s_dim(STRING line, INTEGER *index, struct context_t *context)
{
	STRING temp = NULL;
	STRING name = NULL;
	STRING iname = NULL;
	INTEGER start;
	struct user_defined_t *variable;
	struct type_t const *element_type;
	struct type_t const *array_type;
	INTEGER ubound;
	INTEGER using;
	INTEGER t1;
	INTEGER t2;
	INTEGER t3;
	INTEGER t4;
	INTEGER t5;
	INTEGER t6;

	*index += LEN(r_dim.data);
	skip_spaces(line, index);
	
	for(;;){
		start = *index;
		*index = take_word(line, start);
		if(*index > start){
			string_release(temp);
			temp  = MID(line, start + 1, *index - start);
			string_release(name);
			name = UCASE(temp);
			parse_type_postfix(line, index, context, name, &element_type);
			parse_symbol(line, index, context, '(');
			parse_integer_expression(line, index, context, &ubound);
			parse_symbol(line, index, context, ')');
			array_type = get_type_array_of(&context->type_list, 0, element_type);
			variable = get_user_defined(&context->user_defined_list, name, array_type);
			string_release(iname);
			iname = to_internal_name(variable->name);
			t1 = ++ context->temp;
			t2 = ++ context->temp;
			t3 = ++ context->temp;
			t4 = ++ context->temp;
			t5 = ++ context->temp;
			t6 = ++ context->temp;
			STRING_CONST(template, 
				"\t%tmp# = mul i32 #, %tmp#\r"
				"\t%tmp# = add i32 %tmp#, 8\r"
				"\t%tmp# = call i8* @allocate(i32 %tmp#)\r"
				"\t%tmp# = ptrtoint i8* %tmp# to i32\r"
				"\t%tmp# = add i32 %tmp#, 8\r"
				"\t%tmp# = inttoptr i32 %tmp# to float*\r"
				"\tstore float* %tmp#, float** #\r");
			using = 0;
			using_double(&context->code, template.data, &using, t1);
			using_double(&context->code, template.data, &using, element_type->size);
			using_double(&context->code, template.data, &using, ubound);
			using_double(&context->code, template.data, &using, t2);
			using_double(&context->code, template.data, &using, t1);
			using_double(&context->code, template.data, &using, t3);
			using_double(&context->code, template.data, &using, t2);
			using_double(&context->code, template.data, &using, t4);
			using_double(&context->code, template.data, &using, t3);
			using_double(&context->code, template.data, &using, t5);
			using_double(&context->code, template.data, &using, t4);
			using_double(&context->code, template.data, &using, t6);
			using_double(&context->code, template.data, &using, t5);
			using_double(&context->code, template.data, &using, t6);
			using_string(&context->code, template.data, &using, iname);
			using_rest(&context->code, template.data, &using);
		}else{
			compile_time_error(context->line_number_in_file, S("Variable name required.").data);
		}
		if(line[*index] == ','){
			++ *index;
			skip_spaces(line, index);
		}else{
			break;
		}
	};
	cleanup(context);
	string_release(temp);
	string_release(name);
	string_release(iname);
	return 1;
}

STRING_CONST(r_end, "END");

LOGICAL s_end(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(end_exec, "\ttail call void @END() noreturn\r\tunreachable\r");

	string_append(&context->code, end_exec.data);

	*index += LEN(r_end.data);
	skip_spaces(line, index);
	return 1;
}

STRING_CONST(r_for, "FOR");
STRING_CONST(r_to, "TO");
STRING_CONST(r_step, "STEP");

LOGICAL s_for(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(template_1, 
		"\t%tmp# = call i32* @enter_control(i32 1, i32* null)\r"
		"\tbr label %for#\r"
		"for#:\r"
		"\t%tmp# = call i32 @_setjmp(i32* %tmp#)\r");
		/* load TO, STEP */
	STRING_CONST(template_2,
		"\t%tmp# = add double 1.0, 0.0\r");
	STRING_CONST(template_3,
		"\t%tmp# = icmp eq i32 %tmp#, 0\r"
		"\tbr i1 %tmp#, label %to#, label %next#\r"
		"next#:\r");
		/* load loop variable */
	STRING_CONST(template_4,
		"\t%tmp# = add double %tmp#, %tmp#\r");
		/* store loop variable */
	STRING_CONST(template_5,
		"\tbr label %to#\r"
		"to#:\r");
		/* load loop variable */
	STRING_CONST(template_6,
		"\t%tmp# = fcmp oge double %tmp#, 0.0\r"
		"\t%tmp# = fcmp ole double %tmp#, %tmp#\r"
		"\t%tmp# = fcmp oge double %tmp#, %tmp#\r"
		"\t%tmp# = select i1 %tmp#, i1 %tmp#, i1 %tmp#\r"
		"\tbr i1 %tmp#, label %loop#, label %leave#\r"
		"leave#:\r"
		"\tcall void @leave_control(i32 1, i32* null)\r"
		"\tbr label %exitfor#\r"
		"loop#:\r");

	struct nest_t *nest;
	struct user_defined_t *variable;
	struct expression_t variable_ref;
	struct expression_t expression;
	INTEGER using;
	INTEGER var;
	INTEGER added_var;
	INTEGER jumpbuf;
	INTEGER to_val;
	INTEGER step_val;
	INTEGER setjmp_r;
	INTEGER first_or;
	INTEGER dir;
	INTEGER up;
	INTEGER down;
	INTEGER cond;
	*index += LEN(r_for.data);
	skip_spaces(line, index);
	parse_statement(line, index, context, &variable);
	if(variable == NULL){
		STRING_CONST(message, "Initial counter value assignment required.");
		compile_time_error(context->line_number_in_file, message.data);
	}
	if(! next_word_is(line, *index, context, r_to.data)){
		STRING_CONST(message, "TO required.");
		compile_time_error(context->line_number_in_file, message.data);
	}
	*index += LEN(r_to.data);
	skip_spaces(line, index);
	nest = allocate(sizeof(struct nest_t));
	nest->next = context->nest_list;
	nest->kind = FOR;
	nest->control = variable;
	nest->label = ++ context->temp;
	nest->level = context->level;
	context->nest_list = nest;
	jumpbuf = ++ context->temp;
	setjmp_r = ++ context->temp;
	using = 0;
	using_double(&context->code, template_1.data, &using, jumpbuf);
	using_double(&context->code, template_1.data, &using, nest->label);
	using_double(&context->code, template_1.data, &using, nest->label);
	using_double(&context->code, template_1.data, &using, setjmp_r);
	using_double(&context->code, template_1.data, &using, jumpbuf);
	using_rest  (&context->code, template_1.data, &using);
	parse_double_expression(line, index, context, &to_val);
	if(next_word_is(line, *index, context, r_step.data)){
		*index += LEN(r_step.data);
		skip_spaces(line, index);
		parse_double_expression(line, index, context, &step_val);
	}else{
		step_val = ++ context->temp;
		using = 0;
		using_double(&context->code, template_2.data, &using, step_val);
		using_rest  (&context->code, template_2.data, &using);
	}
	first_or = ++ context->temp;
	using = 0;
	using_double(&context->code, template_3.data, &using, first_or);
	using_double(&context->code, template_3.data, &using, setjmp_r);
	using_double(&context->code, template_3.data, &using, first_or);
	using_double(&context->code, template_3.data, &using, nest->label);
	using_double(&context->code, template_3.data, &using, nest->label);
	using_double(&context->code, template_3.data, &using, nest->label);
	using_rest  (&context->code, template_3.data, &using);
	load_variable(context, variable, &variable_ref);
	to_double(context, &variable_ref, &var);
	cleanup(context);
	added_var = ++ context->temp;
	using = 0;
	using_double(&context->code, template_4.data, &using, added_var);
	using_double(&context->code, template_4.data, &using, var);
	using_double(&context->code, template_4.data, &using, step_val);
	using_rest  (&context->code, template_4.data, &using);
	expression.result = added_var;
	expression.type = &double_type;
	store_variable(context, variable, &expression);
	cleanup(context);
	using = 0;
	using_double(&context->code, template_5.data, &using, nest->label);
	using_double(&context->code, template_5.data, &using, nest->label);
	using_rest  (&context->code, template_5.data, &using);
	load_variable(context, variable, &variable_ref);
	to_double(context, &variable_ref, &var);
	cleanup(context);
	using = 0;
	dir = ++ context->temp;
	up = ++ context->temp;
	down = ++ context->temp;
	cond = ++ context->temp;
	using_double(&context->code, template_6.data, &using, dir);
	using_double(&context->code, template_6.data, &using, step_val);
	using_double(&context->code, template_6.data, &using, up);
	using_double(&context->code, template_6.data, &using, var);
	using_double(&context->code, template_6.data, &using, to_val);
	using_double(&context->code, template_6.data, &using, down);
	using_double(&context->code, template_6.data, &using, var);
	using_double(&context->code, template_6.data, &using, to_val);
	using_double(&context->code, template_6.data, &using, cond);
	using_double(&context->code, template_6.data, &using, dir);
	using_double(&context->code, template_6.data, &using, up);
	using_double(&context->code, template_6.data, &using, down);
	using_double(&context->code, template_6.data, &using, cond);
	using_double(&context->code, template_6.data, &using, nest->label);
	using_double(&context->code, template_6.data, &using, nest->label);
	using_double(&context->code, template_6.data, &using, nest->label);
	using_double(&context->code, template_6.data, &using, nest->label);
	using_double(&context->code, template_6.data, &using, nest->label);
	using_rest  (&context->code, template_6.data, &using);
	return 1;
}

STRING_CONST(r_gosub, "GOSUB");

LOGICAL s_gosub(STRING line, INTEGER *index, struct context_t *context)
{
#ifdef BASIC_CONTROL_STACK
	STRING_CONST(gosub_exec, 
		"\t%tmp# = call i32* @enter_control(i32 3, i32* null)\r"
		"\t%tmp# = call i32 @_setjmp(i32* %tmp#)\r"
		"\t%tmp# = icmp eq i32 %tmp#, 0\r"
		"\tbr i1 %tmp#, label %L#, label %return#\r"
		"return#:\r"
		"\tcall void @leave_control(i32 3, i32* null)\r");
#else
	STRING_CONST(gosub_exec, "\tcall void @program(i32 #)\r");
#endif

	STRING label = NULL;
	STRING temp = NULL;
	INTEGER start;
	INTEGER using;
	struct label_t *label_item;
#ifdef BASIC_CONTROL_STACK
	INTEGER temp_stack = ++ context->temp;
	INTEGER temp_ret = ++ context->temp;
	INTEGER temp_comp = ++ context->temp;
	INTEGER temp_label = ++ context->temp;
#endif
	*index += LEN(r_gosub.data);
	skip_spaces(line, index);
	if(line[*index] == '*'){
		++ *index;
		start = *index;
		*index = take_word(line, *index);
	}else{
		start = *index;
		*index = take_line_number(line, *index);
	}
	if(*index > start){
		temp = MID(line, start + 1, *index - start);
		label = UCASE(temp);
		label_item = get_label(&context->label_list, label, VAL(label));
		if(label_item->state == unused){
			label_item->state = referenced;
			label_item->line_number_in_file = context->line_number_in_file;
		}
#ifdef BASIC_CONTROL_STACK
		using = 0;
		using_double(&context->code, gosub_exec.data, &using, temp_stack);
		using_double(&context->code, gosub_exec.data, &using, temp_ret);
		using_double(&context->code, gosub_exec.data, &using, temp_stack);
		using_double(&context->code, gosub_exec.data, &using, temp_comp);
		using_double(&context->code, gosub_exec.data, &using, temp_ret);
		using_double(&context->code, gosub_exec.data, &using, temp_comp);
		using_string(&context->code, gosub_exec.data, &using, label);
		using_double(&context->code, gosub_exec.data, &using, temp_label);
		using_double(&context->code, gosub_exec.data, &using, temp_label);
		using_rest(&context->code, gosub_exec.data, &using);
#else
		using = 0;
		using_double(&context->code, gosub_exec.data, &using, label_item->number);
		using_rest(&context->code, gosub_exec.data, &using);
		context->jump_table_used = 1;
#endif
		skip_spaces(line, index);
	}
	string_release(label);
	string_release(temp);
	return label != NULL;
}

STRING_CONST(r_goto, "GOTO");

LOGICAL s_goto(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(goto_exec, "\tbr label %L#\r");

	STRING label = NULL;
	STRING temp = NULL;
	INTEGER start;
	INTEGER using;
	struct label_t *label_item;
	*index += LEN(r_goto.data);
	skip_spaces(line, index);

	if(line[*index] == '*'){
		++ *index;
		start = *index;
		*index = take_word(line, *index);
	}else{
		start = *index;
		*index = take_line_number(line, *index);
	}
	if(*index > start){
		temp = MID(line, start + 1, *index - start);
		label = UCASE(temp);
		label_item = get_label(&context->label_list, label, VAL(label));
		if(label_item->state == unused){
			label_item->state = referenced;
			label_item->line_number_in_file = context->line_number_in_file;
		}
		using = 0;
		using_string(&context->code, goto_exec.data, &using, label);
		using_rest(&context->code, goto_exec.data, &using);
		skip_spaces(line, index);
	}
	string_release(label);
	string_release(temp);
	return label != NULL;
}

STRING_CONST(r_if, "IF");
STRING_CONST(r_then, "THEN");
STRING_CONST(r_else, "ELSE");

LOGICAL s_if(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(template_if, "\tbr i1 %tmp#, label %then#, label %else#\r");
	STRING_CONST(template_then, "then#:\r");
	STRING_CONST(template_else, "\tbr label %if#\relse#:\r");
	STRING_CONST(template_endif, "\tbr label %if#\rif#:\r");

	INTEGER using;
	INTEGER cond;
	INTEGER if_label;
	LOGICAL is_goto;
	LOGICAL is_gosub;
	*index += LEN(r_if.data);
	skip_spaces(line, index);
	parse_logical_expression(line, index, context, &cond);
	if_label = ++ context->temp;
	if(next_word_is(line, *index, context, r_then.data)){
		*index += LEN(r_then.data);
		skip_spaces(line, index);
		using = 0;
		using_double(&context->code, template_if.data, &using, cond);
		using_double(&context->code, template_if.data, &using, if_label);
		using_double(&context->code, template_if.data, &using, if_label);
		using_rest(&context->code, template_if.data, &using);
		using = 0;
		using_double(&context->code, template_then.data, &using, if_label);
		using_rest(&context->code, template_then.data, &using);
		++ context->level;
		parse_multi_statement(line, index, context, 0);
		if(context->nest_list != NULL && context->nest_list->level == context->level){
			STRING_CONST(message, "FOR, WHILE without NEXT, WEND.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		-- context->level;
		using = 0;
		using_double(&context->code, template_else.data, &using, if_label);
		using_double(&context->code, template_else.data, &using, if_label);
		using_rest(&context->code, template_else.data, &using);
		if(next_word_is(line, *index, context, r_else.data)){
			*index += LEN(r_else.data);
			skip_spaces(line, index);
			++ context->level;
			parse_multi_statement(line, index, context, 0);
			if(context->nest_list != NULL && context->nest_list->level == context->level){
				STRING_CONST(message, "FOR, WHILE without NEXT, WEND.");
				compile_time_error(context->line_number_in_file, message.data);
			}
			-- context->level;
		}
		using = 0;
		using_double(&context->code, template_endif.data, &using, if_label);
		using_double(&context->code, template_endif.data, &using, if_label);
		using_rest(&context->code, template_endif.data, &using);
	}else{
		is_goto = next_word_is(line, *index, context, r_goto.data);
		is_gosub = ! is_goto && next_word_is(line, *index, context, r_gosub.data);
		using = 0;
		using_double(&context->code, template_if.data, &using, cond);
		using_double(&context->code, template_if.data, &using, if_label);
		using_double(&context->code, template_if.data, &using, if_label);
		using_rest(&context->code, template_if.data, &using);
		using = 0;
		using_double(&context->code, template_then.data, &using, if_label);
		using_rest(&context->code, template_then.data, &using);
		if(is_goto){
			s_goto(line, index, context);
		}else if(is_gosub){
			s_gosub(line, index, context);
		}else{
			STRING_CONST(message, "THEN, GOTO or GOSUB required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		using = 0;
		using_double(&context->code, template_else.data, &using, if_label);
		using_double(&context->code, template_else.data, &using, if_label);
		using_rest(&context->code, template_else.data, &using);
		using = 0;
		using_double(&context->code, template_endif.data, &using, if_label);
		using_double(&context->code, template_endif.data, &using, if_label);
		using_rest(&context->code, template_endif.data, &using);
	}
	return 1;
}

STRING_CONST(r_kill, "KILL");

LOGICAL s_kill(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(template, "\tcall void @KILL(i8* %tmp#)\r");
	
	INTEGER using;
	INTEGER filename;
	*index += LEN(r_kill.data);
	skip_spaces(line, index);
	parse_string_expression(line, index, context, &filename);
	using = 0;
	using_double(&context->code, template.data, &using, filename);
	using_rest(&context->code, template.data, &using);
	cleanup(context);
	return 1;
}

STRING_CONST(r_list, "LIST");

LOGICAL s_list(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(list_exec, "\tcall void @print_string(i32 0, i8* getelementptr(%list_t* @list, i32 0, i32 2, i32 0))\r");

	context->list_used = 1;
	
	string_append(&context->code, list_exec.data);

	*index += LEN(r_list.data);
	skip_spaces(line, index);
	return 1;
}

STRING_CONST(r_next, "NEXT");

LOGICAL s_next(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(back, "\ttail call void @back_control(i32 1, i32* null) noreturn\r");
	STRING_CONST(jump, "\tbr label %for#\r");
	STRING_CONST(exit, "exitfor#:\r");
	STRING_CONST(unknown_pair, "\tunreachable\r");

	struct nest_t *nest;
	INTEGER using;
	LOGICAL abbr = 1;
	INTEGER start;
	struct user_defined_t *variable;
	struct type_t const *type;
	*index += LEN(r_next.data);
	skip_spaces(line, index);
	while(is_word1(line[*index])){
		STRING name = NULL;
		STRING s = NULL;
		abbr = 0;
		start = *index;
		*index = take_word(line, *index);
		s = MID(line, start + 1, *index - start);
		name = UCASE(s);
		variable = query_user_defined(context->user_defined_list, name);
		if(variable == NULL){
			parse_type_postfix(line, index, context, name, &type);
			variable = get_user_defined(&context->user_defined_list, name, type);
		}
		nest = context->nest_list;
		string_append(&context->code, back.data);
		if(nest != NULL && nest->control == variable){
			using = 0;
			using_double(&context->code, jump.data, &using, nest->label);
			using_rest  (&context->code, jump.data, &using);
			if(context->level == nest->level){
				using = 0;
				using_double(&context->code, exit.data, &using, nest->label);
				using_rest  (&context->code, exit.data, &using);
				context->nest_list = nest->next;
				deallocate(nest);
			}
		}else{
			string_append(&context->code, unknown_pair.data);
		}
		string_release(name);
		string_release(s);
		skip_spaces(line, index);
		if(line[*index] != ','){
			break;
		}
		++ *index;
		skip_spaces(line, index);
	}
	if(abbr){
		nest = context->nest_list;
		string_append(&context->code, back.data);
		if(nest != NULL){
			using = 0;
			using_double(&context->code, jump.data, &using, nest->label);
			using_rest  (&context->code, jump.data, &using);
			if(context->level == nest->level){
				using = 0;
				using_double(&context->code, exit.data, &using, nest->label);
				using_rest  (&context->code, exit.data, &using);
				context->nest_list = nest->next;
				deallocate(nest);
			}
		}else{
			string_append(&context->code, unknown_pair.data);
		}
	}
	return 1;
}

STRING_CONST(r_open, "OPEN");
STRING_CONST(r_input, "FOR");
STRING_CONST(r_output, "OUTPUT");
STRING_CONST(r_append, "APPEND");
STRING_CONST(r_as, "AS");

LOGICAL s_open(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(template, "\tcall void @OPEN(i8* %tmp#, i32 %tmp#, i32 %tmp#)\r");
	
	INTEGER filename;
	INTEGER file;
	INTEGER mode;
	INTEGER using;
	*index += LEN(r_open.data);
	skip_spaces(line, index);
	parse_string_expression(line, index, context, &filename);
	if(! next_word_is(line, *index, context, r_for.data)){
		STRING_CONST(message, "FOR required.");
		compile_time_error(context->line_number_in_file, message.data);
	}
	*index += LEN(r_for.data);
	skip_spaces(line, index);
	if(next_word_is(line, *index, context, r_input.data)){
		*index += LEN(r_input.data);
		skip_spaces(line, index);
		mode = FOR_INPUT;
	}else if(next_word_is(line, *index, context, r_output.data)){
		*index += LEN(r_output.data);
		skip_spaces(line, index);
		mode = FOR_OUTPUT;
	}else if(next_word_is(line, *index, context, r_append.data)){
		*index += LEN(r_append.data);
		skip_spaces(line, index);
		mode = FOR_APPEND;
	}else{
		STRING_CONST(message, "File mode required.");
		compile_time_error(context->line_number_in_file, message.data);
	}
	if(! next_word_is(line, *index, context, r_as.data)){
		STRING_CONST(message, "AS required.");
		compile_time_error(context->line_number_in_file, message.data);
	}
	*index += LEN(r_as.data);
	skip_spaces(line, index);
	parse_symbol(line, index, context, '#');
	parse_integer_expression(line, index, context, &file);
	using = 0;
	using_double(&context->code, template.data, &using, filename);
	using_double(&context->code, template.data, &using, mode);
	using_double(&context->code, template.data, &using, file);
	using_rest(&context->code, template.data, &using);
	cleanup(context);
	return 1;
}

STRING_CONST(r_print, "PRINT");
STRING_CONST(r_using, "USING");

LOGICAL s_print(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(stdout, "\t%tmp# = add i32 0, 0\r");
	STRING_CONST(new_line, "\tcall void @print_line(i32 %tmp#)\r");
	STRING_CONST(comma, "\tcall void @print_character(i32 0, i8 9)\r");
	STRING_CONST(dout, "\tcall void @print_double(i32 %tmp#, double %tmp#)\r");
	STRING_CONST(sout, "\tcall void @print_string(i32 %tmp#, i8* %tmp#)\r");
	STRING_CONST(beginusing, "\t%tmp# = alloca i32\r\tstore i32 0, i32* %tmp#\r");
	STRING_CONST(dusing, "\tcall void @print_using_double(i32 %tmp#, i8* %tmp#, i32* %tmp#, double %tmp#)\r");
	STRING_CONST(susing, "\tcall void @print_using_string(i32 %tmp#, i8* %tmp#, i32* %tmp#, i8* %tmp#)\r");
	STRING_CONST(endusing, "\tcall void @print_using_rest(i32 %tmp#, i8* %tmp#, i32* %tmp#)\r");

	INTEGER using;
	INTEGER separated = 1;
	LOGICAL output_new_line = 1;
	INTEGER file;
	INTEGER using_temp = 0;
	struct expression_t format = {0, NULL};
	if(line[*index] == '?'){
		++ *index;
	}else{
		*index += LEN(r_print.data);
	}
	skip_spaces(line, index);
	if(line[*index] == '#'){
		++ *index;
		skip_spaces(line, index);
		parse_integer_expression(line, index, context, &file);
		parse_symbol(line, index, context, ',');
	}else{
		file = ++ context->temp;
		using = 0;
		using_double(&context->code, stdout.data, &using, file);
		using_rest(&context->code, stdout.data, &using);
	}
	if(next_word_is(line, *index, context, r_using.data)){
		*index += LEN(r_using.data);
		skip_spaces(line, index);
		if(! parse_expression(line, index, context, &format, 0) || format.type != &string_type){
			STRING_CONST(message, "Format required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		using_temp = ++ context->temp;
		using = 0;
		using_double(&context->code, beginusing.data, &using, using_temp);
		using_double(&context->code, beginusing.data, &using, using_temp);
		using_rest(&context->code, beginusing.data, &using);
		parse_symbol(line, index, context, ';');
	}
	for(;;){
		struct expression_t expression = {0, NULL};
		if(line[*index] == ','){
			++ *index;
			skip_spaces(line, index);
			string_append(&context->code, comma.data);
			output_new_line = 0;
			separated = 1;
		}else if(line[*index] == ';'){
			++ *index;
			skip_spaces(line, index);
			output_new_line = 0;
			separated = 1;
		}else if(separated && parse_expression(line, index, context, &expression, 0)){
			if(expression.type == &logical_type || expression.type == &integer_type 
				|| expression.type == &single_type || expression.type == &double_type)
			{
				INTEGER value;
				to_double(context, &expression, &value);
				if(format.result != 0){
					using = 0;
					using_double(&context->code, dusing.data, &using, file);
					using_double(&context->code, dusing.data, &using, format.result);
					using_double(&context->code, dusing.data, &using, using_temp);
					using_double(&context->code, dusing.data, &using, value);
					using_rest(&context->code, dusing.data, &using);
				}else{
					using = 0;
					using_double(&context->code, dout.data, &using, file);
					using_double(&context->code, dout.data, &using, value);
					using_rest(&context->code, dout.data, &using);
				}
			}else if(expression.type == &string_type){
				if(format.result != 0){
					using = 0;
					using_double(&context->code, susing.data, &using, file);
					using_double(&context->code, susing.data, &using, format.result);
					using_double(&context->code, susing.data, &using, using_temp);
					using_double(&context->code, susing.data, &using, expression.result);
					using_rest(&context->code, susing.data, &using);
				}else{
					using = 0;
					using_double(&context->code, sout.data, &using, file);
					using_double(&context->code, sout.data, &using, expression.result);
					using_rest(&context->code, sout.data, &using);
				}
			}else{
				STRING_CONST(message, "Type mismatch.");
				compile_time_error(context->line_number_in_file, message.data);
			}
			output_new_line = 1;
			separated = 0;
		}else{
			break;
		}
	}
	if(format.result != 0){
		using = 0;
		using_double(&context->code, endusing.data, &using, file);
		using_double(&context->code, endusing.data, &using, format.result);
		using_double(&context->code, endusing.data, &using, using_temp);
		using_rest(&context->code, endusing.data, &using);
	}
	if(output_new_line){
		using = 0;
		using_double(&context->code, new_line.data, &using, file);
		using_rest(&context->code, new_line.data, &using);
	}
	cleanup(context);
	return 1;
}

STRING_CONST(r_read, "READ");

LOGICAL s_read(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(at, "@");
	INTEGER using;
	INTEGER start;
	struct user_defined_t *variable;
	struct type_t const *type;
	*index += LEN(r_read.data);
	skip_spaces(line, index);
	while(is_word1(line[*index])){
		STRING name = NULL;
		STRING s = NULL;
		start = *index;
		*index = take_word(line, *index);
		s = MID(line, start + 1, *index - start);
		name = UCASE(s);
		variable = query_user_defined(context->user_defined_list, name);
		if(variable == NULL){
			parse_type_postfix(line, index, context, name, &type);
			variable = get_user_defined(&context->user_defined_list, name, type);
		}
		if(variable->type == &string_type){
			STRING_CONST(template, "\tcall void @read_string(i8*** @data_reading, i8** @)\r");
			STRING name = to_internal_name(variable->name);
			using = 0;
			using_string(&context->code, template.data, &using, at.data);
			using_string(&context->code, template.data, &using, at.data);
			using_string(&context->code, template.data, &using, name);
			using_rest  (&context->code, template.data, &using);
			string_release(name);
		}else{
			STRING_CONST(template, "\t%tmp# = alloca double\r\tcall void @read_double(i8*** @data_reading, double* %tmp#)\r");
			struct expression_t expression;
			expression.result = ++ context->temp;
			expression.type = &double_type;
			using = 0;
			using_double(&context->code, template.data, &using, expression.result);
			using_double(&context->code, template.data, &using, expression.result);
			using_rest  (&context->code, template.data, &using);
			store_variable(context, variable, &expression);
		}
		string_release(name);
		string_release(s);
		skip_spaces(line, index);
		if(line[*index] != ','){
			break;
		}
		++ *index;
		skip_spaces(line, index);
	}
	return 1;
}

STRING_CONST(r_restore, "RESTORE");

LOGICAL s_restore(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(template, "\tcall void @restore(i32 #)\r");

	STRING label = NULL;
	STRING temp = NULL;
	INTEGER start;
	INTEGER using;
	struct label_t *label_item;
	*index += LEN(r_restore.data);
	skip_spaces(line, index);

	if(line[*index] == '*'){
		++ *index;
		start = *index;
		*index = take_word(line, *index);
		if(*index == start){
			STRING_CONST(message, "Invalid label.");
			compile_time_error(context->line_number_in_file, message.data);
		}
	}else{
		start = *index;
		*index = take_line_number(line, *index);
	}
	if(*index > start){
		temp = MID(line, start + 1, *index - start);
		label = UCASE(temp);
		label_item = get_label(&context->label_list, label, VAL(label));
		if(label_item->state == unused){
			label_item->state = referenced;
			label_item->line_number_in_file = context->line_number_in_file;
		}
		using = 0;
		using_double(&context->code, template.data, &using, label_item->number);
		using_rest  (&context->code, template.data, &using);
		skip_spaces(line, index);
	}else{
		using = 0;
		using_double(&context->code, template.data, &using, 0);
		using_rest  (&context->code, template.data, &using);
	}
	string_release(label);
	string_release(temp);
	return 1;
}

STRING_CONST(r_return, "RETURN");

LOGICAL s_return(STRING line, INTEGER *index, struct context_t *context)
{
#ifdef BASIC_CONTROL_STACK
	STRING_CONST(return_exec, "\ttail call void @back_control(i32 3, i32* null) noreturn\r");
#else
	STRING_CONST(return_exec, "\tret void\r");
#endif

	string_append(&context->code, return_exec.data);

	*index += LEN(r_return.data);
	skip_spaces(line, index);
	return 1;
}

STRING_CONST(r_wend, "WEND");

LOGICAL s_wend(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(back, "\ttail call void @back_control(i32 2, i32* null) noreturn\r");
	STRING_CONST(jump, "\tbr label %while#\r");
	STRING_CONST(exit, "exitwhile#:\r");
	STRING_CONST(unknown_pair, "\tunreachable\r");

	struct nest_t *nest;
	INTEGER using;
	*index += LEN(r_wend.data);
	skip_spaces(line, index);
	nest = context->nest_list;
	string_append(&context->code, back.data);
	if(nest != NULL){
		using = 0;
		using_double(&context->code, jump.data, &using, nest->label);
		using_rest  (&context->code, jump.data, &using);
		if(context->level == nest->level){
			using = 0;
			using_double(&context->code, exit.data, &using, nest->label);
			using_rest  (&context->code, exit.data, &using);
			/* end this while-loop */
			context->nest_list = nest->next;
			deallocate(nest);
		}
	}else{
		string_append(&context->code, unknown_pair.data);
	}
	return 1;
}

STRING_CONST(r_while, "WHILE");

LOGICAL s_while(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(b_template, 
		"\t%tmp# = call i32* @enter_control(i32 2, i32* null)\r"
		"\tbr label %while#\r"
		"while#:\r"
		"\t%tmp# = call i32 @_setjmp(i32* %tmp#)\r");
	STRING_CONST(e_template,
		"\tbr i1 %tmp#, label %loop#, label %wend#\r"
		"wend#:\r"
		"\tcall void @leave_control(i32 2, i32* null)\r"
		"\tbr label %exitwhile#\r"
		"loop#:\r");

	struct nest_t *nest;
	INTEGER using;
	INTEGER buf;
	INTEGER cond;
	*index += LEN(r_while.data);
	skip_spaces(line, index);
	buf = ++ context->temp;
	nest = allocate(sizeof(struct nest_t));
	nest->next = context->nest_list;
	nest->kind = WHILE;
	nest->control = NULL;
	nest->label = ++ context->temp;
	nest->level = context->level;
	context->nest_list = nest;
	using = 0;
	using_double(&context->code, b_template.data, &using, buf);
	using_double(&context->code, b_template.data, &using, nest->label);
	using_double(&context->code, b_template.data, &using, nest->label);
	using_double(&context->code, b_template.data, &using, ++ context->temp);
	using_double(&context->code, b_template.data, &using, buf);
	using_rest  (&context->code, b_template.data, &using);
	parse_logical_expression(line, index, context, &cond);
	using = 0;
	using_double(&context->code, e_template.data, &using, cond);
	using_double(&context->code, e_template.data, &using, nest->label);
	using_double(&context->code, e_template.data, &using, nest->label);
	using_double(&context->code, e_template.data, &using, nest->label);
	using_double(&context->code, e_template.data, &using, nest->label);
	using_double(&context->code, e_template.data, &using, nest->label);
	using_rest  (&context->code, e_template.data, &using);
	return 1;
}

STRING_CONST(r_write, "WRITE");

LOGICAL s_write(STRING line, INTEGER *index, struct context_t *context)
{
	STRING_CONST(stdout, "\t%tmp# = add i32 0, 0\r");
	STRING_CONST(new_line, "\tcall void @print_line(i32 %tmp#)\r");
	STRING_CONST(comma, "\tcall void @print_character(i32 0, i8 44)\r");
	STRING_CONST(dout, "\tcall void @print_double(i32 %tmp#, double %tmp#)\r");
	STRING_CONST(sout, "\tcall void @write_string(i32 %tmp#, i8* %tmp#)\r");

	INTEGER using;
	INTEGER separated = 1;
	LOGICAL output_new_line = 1;
	INTEGER file;
	*index += LEN(r_write.data);
	skip_spaces(line, index);
	if(line[*index] == '#'){
		++ *index;
		skip_spaces(line, index);
		parse_integer_expression(line, index, context, &file);
		parse_symbol(line, index, context, ',');
	}else{
		file = ++ context->temp;
		using = 0;
		using_double(&context->code, stdout.data, &using, file);
		using_rest(&context->code, stdout.data, &using);
	}
	for(;;){
		struct expression_t expression = {0, NULL};
		if(line[*index] == ','){
			++ *index;
			skip_spaces(line, index);
			string_append(&context->code, comma.data);
			output_new_line = 0;
			separated = 1;
		}else if(separated && parse_expression(line, index, context, &expression, 0)){
			if(expression.type == &logical_type || expression.type == &integer_type 
				|| expression.type == &single_type || expression.type == &double_type)
			{
				INTEGER value;
				to_double(context, &expression, &value);
				using = 0;
				using_double(&context->code, dout.data, &using, file);
				using_double(&context->code, dout.data, &using, value);
				using_rest(&context->code, dout.data, &using);
			}else if(expression.type == &string_type){
				using = 0;
				using_double(&context->code, sout.data, &using, file);
				using_double(&context->code, sout.data, &using, expression.result);
				using_rest(&context->code, sout.data, &using);
			}else{
				STRING_CONST(message, "Type mismatch.");
				compile_time_error(context->line_number_in_file, message.data);
			}
			output_new_line = 1;
			separated = 0;
		}else{
			break;
		}
	}
	if(output_new_line){
		using = 0;
		using_double(&context->code, new_line.data, &using, file);
		using_rest(&context->code, new_line.data, &using);
	}
	cleanup(context);
	return 1;
}

struct statement_t {
	STRING keyword;
	LOGICAL (*func)(STRING line, INTEGER *index, struct context_t *context);
} statements[] = {
	{ r_close.data, s_close },
	{ r_data.data, s_data },
	{ r_defdbl.data, s_deftype },
	{ r_defint.data, s_deftype },
	{ r_defsng.data, s_deftype },
	{ r_defstr.data, s_deftype },
	{ r_dim.data, s_dim },
	{ r_end.data, s_end },
	{ r_for.data, s_for },
	{ r_gosub.data, s_gosub },
	{ r_goto.data, s_goto },
	{ r_if.data, s_if },
	{ r_kill.data, s_kill },
	{ r_list.data, s_list },
	{ r_next.data, s_next },
	{ r_open.data, s_open },
	{ r_print.data, s_print },
	{ r_read.data, s_read },
	{ r_restore.data, s_restore },
	{ r_return.data, s_return },
	{ r_wend.data, s_wend },
	{ r_while.data, s_while },
	{ r_write.data, s_write },
};

enum { statement_count = sizeof(statements) / sizeof(struct statement_t) };

/* compiler body */

void parse_type_postfix(STRING line, INTEGER *index, struct context_t *context, STRING name, struct type_t const **type)
{
	if(line[*index] == '%'){
		++ *index;
		skip_spaces(line, index);
		*type = &integer_type;
	}else if(line[*index] == '!'){
		++ *index;
		skip_spaces(line, index);
		*type = &single_type;
	}else if(line[*index] == '#'){
		++ *index;
		skip_spaces(line, index);
		*type = &double_type;
	}else if(line[*index] == '$'){
		++ *index;
		skip_spaces(line, index);
		*type = &string_type;
	}else{
		*type = context->default_type[name[0] - 'A'];
	}
}

void load_variable(struct context_t *context, struct user_defined_t *variable, struct expression_t *expression)
{
	STRING name = NULL;
	INTEGER using;
	struct temp_in_expression_t *item;
	item = allocate(sizeof(struct temp_in_expression_t));
	item->next = context->temp_list;
	item->temp = ++ context->temp;
	item->type = variable->type;
	context->temp_list = item;
	expression->result = item->temp;
	expression->type = item->type;
	name = to_internal_name(variable->name);
	if(variable->type == &integer_type){
		STRING_CONST(iload, "\t%tmp# = load i32* #\r");
		using = 0;
		using_double(&context->code, iload.data, &using, expression->result);
		using_string(&context->code, iload.data, &using, name);
		using_rest(&context->code, iload.data, &using);
	}else if(variable->type == &single_type){
		STRING_CONST(fload, "\t%tmp# = load float* #\r");
		using = 0;
		using_double(&context->code, fload.data, &using, expression->result);
		using_string(&context->code, fload.data, &using, name);
		using_rest(&context->code, fload.data, &using);
	}else if(variable->type == &double_type){
		STRING_CONST(dload, "\t%tmp# = load double* #\r");
		using = 0;
		using_double(&context->code, dload.data, &using, expression->result);
		using_string(&context->code, dload.data, &using, name);
		using_rest(&context->code, dload.data, &using);
	}else{
		STRING_CONST(sload, "\t%tmp# = load i8** #\r\tcall void @string_addref(i8* %tmp#)\r");
		using = 0;
		using_double(&context->code, sload.data, &using, expression->result);
		using_string(&context->code, sload.data, &using, name);
		using_double(&context->code, sload.data, &using, expression->result);
		using_rest(&context->code, sload.data, &using);
	}
	string_release(name);
}

LOGICAL parse_term(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	STRING_CONST(iload, "\t%tmp# = add i32 #, 0\r");
	STRING_CONST(sload, "\t%tmp# = add float#0.#, 0.0\r");
	STRING_CONST(dload, "\t%tmp# = add double#0.#, 0.0\r");
	STRING_CONST(stringload, "\t%tmp# = getelementptr %string#_t* @string#, i32 0, i32 2, i32 0\r");
	
	STRING s = NULL;
	STRING func = NULL;
	LOGICAL result;
	INTEGER start;
	INTEGER using;
	INTEGER ivalue;
	DOUBLE fvalue;
	LOGICAL floating;
	LOGICAL is_double;
	struct temp_in_expression_t *item;
	struct string_literal_t *literal;
	struct type_t const *type;
	struct user_defined_t *user_defined;
	if(line[*index] == '('){
		++ *index;
		skip_spaces(line, index);
		result = parse_expression(line, index, context, expression, 1);
		if(line[*index] != ')'){
			compile_time_error(context->line_number_in_file, S("\")\" required.").data);
		}
		++ *index;
		skip_spaces(line, index);
	}else if(line[*index] == '&'){
		start = *index;
		++ *index;
		switch(line[*index]){
		case 'o': case 'O':
			do{
				++ *index;
			}while(line[*index] >= '0' && line[*index] <= '7');
			break;
		case 'h': case 'H':
			do{
				++ *index;
			}while((line[*index] >= '0' && line[*index] <= '9') 
				|| (line[*index] >= 'A' && line[*index] <= 'F') 
				|| (line[*index] >= 'a' && line[*index] <= 'f'));
			break;
		default:
			compile_time_error(context->line_number_in_file, S("Bad format for hexadecimal or octal number.").data);
		}
		s = MID(line, start + 1, *index - start);
		if(line[*index] == '%'){
			++ *index;
		}
		skip_spaces(line, index);
		ivalue = VAL(s);
		item = allocate(sizeof(struct temp_in_expression_t));
		item->next = context->temp_list;
		item->temp = ++ context->temp;
		item->type = &integer_type;
		context->temp_list = item;
		expression->result = item->temp;
		expression->type = item->type;
		using = 0;
		using_double(&context->code, iload.data, &using, item->temp);
		using_double(&context->code, iload.data, &using, ivalue);
		using_rest(&context->code, iload.data, &using);
		result = 1;
	}else if(line[*index] >= '0' && line[*index] <= '9'){
		start = *index;
		do{
			++ *index;
		}while(line[*index] >= '0' && line[*index] <= '9');
		floating = 0;
		if(line[*index] == '.'){
			floating = 1;
			++ *index;
			while(line[*index] >= '0' && line[*index] <= '9'){
				++ *index;
			}
		}
		s = MID(line, start + 1, *index - start);
		fvalue = VAL(s);
		ivalue = fvalue;
		if(floating || line[*index] == '!' || line[*index] == '#'){
			is_double = 0;
			if(line[*index] == '!'){
				++ *index;
			}else if(line[*index] == '#'){
				++ *index;
				is_double = 1;
			}
			skip_spaces(line, index);
			if(is_double){
				item = allocate(sizeof(struct temp_in_expression_t));
				item->next = context->temp_list;
				item->temp = ++ context->temp;
				item->type = &double_type;
				context->temp_list = item;
				expression->result = item->temp;
				expression->type = item->type;
				using = 0;
				using_double(&context->code, dload.data, &using, item->temp);
				using_double(&context->code, dload.data, &using, fvalue);
				using_rest(&context->code, dload.data, &using);
			}else{
				item = allocate(sizeof(struct temp_in_expression_t));
				item->next = context->temp_list;
				item->temp = ++ context->temp;
				item->type = &single_type;
				context->temp_list = item;
				expression->result = item->temp;
				expression->type = item->type;
				using = 0;
				using_double(&context->code, sload.data, &using, item->temp);
				using_double(&context->code, sload.data, &using, fvalue);
				using_rest(&context->code, sload.data, &using);
			}
		}else{
			if(line[*index] == '%'){
				++ *index;
			}
			skip_spaces(line, index);
			item = allocate(sizeof(struct temp_in_expression_t));
			item->next = context->temp_list;
			item->temp = ++ context->temp;
			item->type = &integer_type;
			context->temp_list = item;
			expression->result = item->temp;
			expression->type = item->type;
			using = 0;
			using_double(&context->code, iload.data, &using, item->temp);
			using_double(&context->code, iload.data, &using, ivalue);
			using_rest(&context->code, iload.data, &using);
		}
		result = 1;
	}else if(line[*index] == '\"'){
		s = take_string_literal(line, index);
		literal = allocate(sizeof(struct string_literal_t));
		literal->next = context->string_literal_list;
		literal->number = (context->string_literal_list != NULL) ? context->string_literal_list->number + 1 : 1;
		literal->value = s;
		string_addref(literal->value);
		context->string_literal_list = literal;
		item = allocate(sizeof(struct temp_in_expression_t));
		item->next = context->temp_list;
		item->temp = ++ context->temp;
		item->type = &string_type;
		context->temp_list = item;
		expression->result = item->temp;
		expression->type = item->type;
		using = 0;
		using_double(&context->code, stringload.data, &using, item->temp);
		using_double(&context->code, stringload.data, &using, literal->number);
		using_double(&context->code, stringload.data, &using, literal->number);
		using_rest(&context->code, stringload.data, &using);
		skip_spaces(line, index);
		result = 1;
	}else{
		/* function call or variable */
		start = *index;
		*index = take_word(line, start);
		if(*index == start){
			result = 0;
		}else{
			string_release(s);
			s = MID(line, start + 1, *index - start);
			string_release(func);
			func = UCASE(s);
			user_defined = query_user_defined(context->user_defined_list, func);
			if(user_defined == NULL){
				/* search internal function ... not implemented */
			}
			if(user_defined == NULL){
				parse_type_postfix(line, index, context, func, &type);
				user_defined = get_user_defined(&context->user_defined_list, func, type);
			}
			load_variable(context, user_defined, expression);
			skip_spaces(line, index);
			result = 1;
		}
	}
	string_release(s);
	string_release(func);
	return result;
}

LOGICAL parse_power(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	LOGICAL result = parse_term(line, index, context, expression);
	if(result && line[*index] == '^'){
		STRING template = NULL;
		INTEGER left;
		INTEGER right;
		INTEGER using;
		struct expression_t right_expression;
		++ *index;
		skip_spaces(line, index);
		if(! parse_power(line, index, context, &right_expression)){ /* ^ op. is right association */
			STRING_CONST(message, "Right term required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		if(expression->type == &integer_type && right_expression.type == &integer_type){
			STRING_CONST(i_template, "\t%tmp# = call i32 @power_integer(i32 %tmp#, i32 %tmp#)\r");
			template = i_template.data;
			to_integer(context, expression, &left);
			to_integer(context, &right_expression, &right);
			expression->type = &integer_type;
		}else if((expression->type == &single_type || expression->type == &integer_type)
			&& (right_expression.type == &single_type || right_expression.type == &integer_type))
		{
			STRING_CONST(f_template, "\t%tmp# = call float @power_single(float %tmp#, float %tmp#)\r");
			template = f_template.data;
			to_single(context, expression, &left);
			to_single(context, &right_expression, &right);
			expression->type = &single_type;
		}else{
			STRING_CONST(d_template, "\t%tmp# = call double @power_double(double %tmp#, double %tmp#)\r");
			template = d_template.data;
			to_double(context, expression, &left);
			to_double(context, &right_expression, &right);
			expression->type = &double_type;
		}
		expression->result = ++ context->temp;
		using = 0;
		using_double(&context->code, template, &using, expression->result);
		using_double(&context->code, template, &using, left);
		using_double(&context->code, template, &using, right);
		using_rest(&context->code, template, &using);
		string_release(template);
	}
	return result;
}

LOGICAL parse_plus_minus(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	LOGICAL result;
	if(line[*index] == '+'){
		++ *index;
		skip_spaces(line, index);
		result = parse_power(line, index, context, expression);
		if(expression->type == &string_type){
			STRING_CONST(message, "Type mismatch.");
			compile_time_error(context->line_number_in_file, message.data);
		}
	}else if(line[*index] == '-'){
		STRING_CONST(template, "\t%tmp# = sub double 0.0, %tmp#\r");
		INTEGER x;
		INTEGER using;
		++ *index;
		skip_spaces(line, index);
		result = parse_power(line, index, context, expression);
		to_double(context, expression, &x);
		expression->result = ++ context->temp;
		expression->type = &double_type;
		using = 0;
		using_double(&context->code, template.data, &using, expression->result);
		using_double(&context->code, template.data, &using, x);
		using_rest(&context->code, template.data, &using);
	}else{
		result = parse_power(line, index, context, expression);
	}
	return result;
}

LOGICAL parse_mul_fra(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	LOGICAL result = parse_plus_minus(line, index, context, expression);
	while(result){
		STRING template = NULL;
		if(line[*index] == '*'){
			STRING_CONST(add_template, "\t%tmp# = mul double %tmp#, %tmp#\r");
			template = add_template.data;
		}else if(line[*index] == '/'){
			STRING_CONST(sub_template, "\t%tmp# = fdiv double %tmp#, %tmp#\r");
			template = sub_template.data;
		}else{
			break;
		}
		INTEGER left;
		INTEGER right;
		INTEGER using;
		struct expression_t right_expression;
		++ *index;
		skip_spaces(line, index);
		to_double(context, expression, &left);
		if(! parse_plus_minus(line, index, context, &right_expression)){
			STRING_CONST(message, "Right term required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		to_double(context, &right_expression, &right);
		expression->result = ++ context->temp;
		expression->type = &double_type;
		using = 0;
		using_double(&context->code, template, &using, expression->result);
		using_double(&context->code, template, &using, left);
		using_double(&context->code, template, &using, right);
		using_rest(&context->code, template, &using);
		string_release(template);
	}
	return result;
}

LOGICAL parse_div(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	LOGICAL result = parse_mul_fra(line, index, context, expression);
	while(result && line[*index] == '\\'){
		STRING_CONST(template, "\t%tmp# = sdiv i32 %tmp#, %tmp#\r");
		INTEGER left;
		INTEGER right;
		INTEGER using;
		struct expression_t right_expression;
		++ *index;
		skip_spaces(line, index);
		to_integer(context, expression, &left);
		if(! parse_mul_fra(line, index, context, &right_expression)){
			STRING_CONST(message, "Right term required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		to_integer(context, &right_expression, &right);
		expression->result = ++ context->temp;
		expression->type = &integer_type;
		using = 0;
		using_double(&context->code, template.data, &using, expression->result);
		using_double(&context->code, template.data, &using, left);
		using_double(&context->code, template.data, &using, right);
		using_rest(&context->code, template.data, &using);
	}
	return result;
}

STRING_CONST(r_mod, "MOD");

LOGICAL parse_mod(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	LOGICAL result = parse_div(line, index, context, expression);
	while(result && next_word_is(line, *index, context, r_mod.data)){
		STRING_CONST(template, "\t%tmp# = srem i32 %tmp#, %tmp#\r");
		INTEGER left;
		INTEGER right;
		INTEGER using;
		struct expression_t right_expression;
		*index += LEN(r_mod.data);
		skip_spaces(line, index);
		to_integer(context, expression, &left);
		if(! parse_div(line, index, context, &right_expression)){
			STRING_CONST(message, "Right term required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		to_integer(context, &right_expression, &right);
		expression->result = ++ context->temp;
		expression->type = &integer_type;
		using = 0;
		using_double(&context->code, template.data, &using, expression->result);
		using_double(&context->code, template.data, &using, left);
		using_double(&context->code, template.data, &using, right);
		using_rest(&context->code, template.data, &using);
	}
	return result;
}

LOGICAL parse_add_sub(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	LOGICAL result = parse_mod(line, index, context, expression);
	while(result){
		INTEGER op;
		STRING template = NULL;
		if(line[*index] == '+'){
			STRING_CONST(add_template, "\t%tmp# = add double %tmp#, %tmp#\r");
			template = add_template.data;
			op = 1;
		}else if(line[*index] == '-'){
			STRING_CONST(sub_template, "\t%tmp# = sub double %tmp#, %tmp#\r");
			template = sub_template.data;
			op = 2;
		}else{
			break;
		}
		INTEGER left;
		INTEGER right;
		INTEGER using;
		struct expression_t right_expression;
		++ *index;
		skip_spaces(line, index);
		if(! parse_div(line, index, context, &right_expression)){
			STRING_CONST(message, "Right term required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		if(op == 1 && expression->type == &string_type && right_expression.type == &string_type){
			STRING_CONST(cat_template, "\t%tmp# = call i8* @string_concatenate(i8* %tmp#, i8* %tmp#)\r");
			left = expression->result;
			right = right_expression.result;
			struct temp_in_expression_t *item;
			item = allocate(sizeof(struct temp_in_expression_t));
			item->next = context->temp_list;
			item->temp = ++ context->temp;
			item->type = &string_type;
			context->temp_list = item;
			expression->result = item->temp;
			expression->type = item->type;
			using = 0;
			using_double(&context->code, cat_template.data, &using, expression->result);
			using_double(&context->code, cat_template.data, &using, left);
			using_double(&context->code, cat_template.data, &using, right);
			using_rest(&context->code, cat_template.data, &using);
		}else{
			to_double(context, expression, &left);
			to_double(context, &right_expression, &right);
			expression->result = ++ context->temp;
			expression->type = &double_type;
			using = 0;
			using_double(&context->code, template, &using, expression->result);
			using_double(&context->code, template, &using, left);
			using_double(&context->code, template, &using, right);
			using_rest(&context->code, template, &using);
		}
		string_release(template);
	}
	return result;
}

LOGICAL parse_compare(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	STRING_CONST(strcomp, "\t%tmp# = call i32 @string_compare(i8* %tmp#, i8* %tmp#)\r");
	
	LOGICAL result = parse_add_sub(line, index, context, expression);
	while(result){
		STRING d_template = NULL;
		STRING s_template = NULL;
		if(line[*index] == '='){
			STRING_CONST(eq_d_template, "\t%tmp# = fcmp oeq double %tmp#, %tmp#\r");
			STRING_CONST(eq_s_template, "\t%tmp# = icmp eq i32 %tmp#, 0\r");
			d_template = eq_d_template.data;
			s_template = eq_s_template.data;
			++ *index;
		}else if(line[*index] == '<' && line[*index + 1] == '='){
			STRING_CONST(le_d_template, "\t%tmp# = fcmp ole double %tmp#, %tmp#\r");
			STRING_CONST(le_s_template, "\t%tmp# = icmp sle i32 %tmp#, 0\r");
			d_template = le_d_template.data;
			s_template = le_s_template.data;
			*index += 2;
		}else if(line[*index] == '>' && line[*index + 1] == '='){
			STRING_CONST(ge_d_template, "\t%tmp# = fcmp oge double %tmp#, %tmp#\r");
			STRING_CONST(ge_s_template, "\t%tmp# = icmp sge i32 %tmp#, 0\r");
			d_template = ge_d_template.data;
			s_template = ge_s_template.data;
			*index += 2;
		}else if(line[*index] == '<' && line[*index + 1] == '>'){
			STRING_CONST(ne_d_template, "\t%tmp# = fcmp one double %tmp#, %tmp#\r");
			STRING_CONST(ne_s_template, "\t%tmp# = icmp ne i32 %tmp#, 0\r");
			d_template = ne_d_template.data;
			s_template = ne_s_template.data;
			*index += 2;
		}else if(line[*index] == '<'){
			STRING_CONST(lt_d_template, "\t%tmp# = fcmp olt double %tmp#, %tmp#\r");
			STRING_CONST(lt_s_template, "\t%tmp# = icmp slt i32 %tmp#, 0\r");
			d_template = lt_d_template.data;
			s_template = lt_s_template.data;
			++ *index;
		}else if(line[*index] == '>'){
			STRING_CONST(gt_d_template, "\t%tmp# = fcmp ogt double %tmp#, %tmp#\r");
			STRING_CONST(gt_s_template, "\t%tmp# = icmp sgt i32 %tmp#, 0\r");
			d_template = gt_d_template.data;
			s_template = gt_s_template.data;
			++ *index;
		}else{
			break;
		}
		INTEGER left;
		INTEGER right;
		INTEGER using;
		struct expression_t right_expression;
		skip_spaces(line, index);
		if(! parse_div(line, index, context, &right_expression)){
			STRING_CONST(message, "Right term required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		if(expression->type == &string_type && right_expression.type == &string_type){
			INTEGER compared = ++ context->temp;
			using = 0;
			using_double(&context->code, strcomp.data, &using, compared);
			using_double(&context->code, strcomp.data, &using, expression->result);
			using_double(&context->code, strcomp.data, &using, right_expression.result);
			using_rest  (&context->code, strcomp.data, &using);
			expression->result = ++ context->temp;
			expression->type = &logical_type;
			using = 0;
			using_double(&context->code, s_template, &using, expression->result);
			using_double(&context->code, s_template, &using, compared);
			using_rest  (&context->code, s_template, &using);
		}else{
			to_double(context, expression, &left);
			to_double(context, &right_expression, &right);
			expression->result = ++ context->temp;
			expression->type = &logical_type;
			using = 0;
			using_double(&context->code, d_template, &using, expression->result);
			using_double(&context->code, d_template, &using, left);
			using_double(&context->code, d_template, &using, right);
			using_rest  (&context->code, d_template, &using);
		}
		string_release(d_template);
		string_release(s_template);
	}
	return result;
}

STRING_CONST(r_not, "NOT");

LOGICAL parse_not(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	LOGICAL result;
	if(next_word_is(line, *index, context, r_not.data)){
		STRING_CONST(template, "\t%tmp# = xor i32 %tmp#, 4294967295\r");
		INTEGER x;
		INTEGER using;
		*index += LEN(r_not.data);
		skip_spaces(line, index);
		result = parse_compare(line, index, context, expression);
		to_integer(context, expression, &x);
		expression->result = ++ context->temp;
		expression->type = &integer_type;
		using = 0;
		using_double(&context->code, template.data, &using, expression->result);
		using_double(&context->code, template.data, &using, x);
		using_rest(&context->code, template.data, &using);
	}else{
		result = parse_compare(line, index, context, expression);
	}
	return result;
}

STRING_CONST(r_and, "AND");

LOGICAL parse_and(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	LOGICAL result = parse_not(line, index, context, expression);
	while(result && next_word_is(line, *index, context, r_mod.data)){
		STRING_CONST(template, "\t%tmp# = and i32 %tmp#, %tmp#\r");
		INTEGER left;
		INTEGER right;
		INTEGER using;
		struct expression_t right_expression;
		*index += LEN(r_and.data);
		skip_spaces(line, index);
		to_integer(context, expression, &left);
		if(! parse_not(line, index, context, &right_expression)){
			STRING_CONST(message, "Right term required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		to_integer(context, &right_expression, &right);
		expression->result = ++ context->temp;
		expression->type = &integer_type;
		using = 0;
		using_double(&context->code, template.data, &using, expression->result);
		using_double(&context->code, template.data, &using, left);
		using_double(&context->code, template.data, &using, right);
		using_rest(&context->code, template.data, &using);
	}
	return result;
}

STRING_CONST(r_or, "OR");

LOGICAL parse_or(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	LOGICAL result = parse_and(line, index, context, expression);
	while(result && next_word_is(line, *index, context, r_mod.data)){
		STRING_CONST(template, "\t%tmp# = and i32 %tmp#, %tmp#\r");
		INTEGER left;
		INTEGER right;
		INTEGER using;
		struct expression_t right_expression;
		*index += LEN(r_or.data);
		skip_spaces(line, index);
		to_integer(context, expression, &left);
		if(! parse_and(line, index, context, &right_expression)){
			STRING_CONST(message, "Right term required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		to_integer(context, &right_expression, &right);
		expression->result = ++ context->temp;
		expression->type = &integer_type;
		using = 0;
		using_double(&context->code, template.data, &using, expression->result);
		using_double(&context->code, template.data, &using, left);
		using_double(&context->code, template.data, &using, right);
		using_rest(&context->code, template.data, &using);
	}
	return result;
}

STRING_CONST(r_eqv, "EQV");
STRING_CONST(r_xor, "XOR");
STRING_CONST(r_imp, "IMP");

LOGICAL parse_eqv_xor_imp(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression)
{
	LOGICAL result = parse_or(line, index, context, expression);
	while(result){
		STRING template = NULL;
		INTEGER op;
		if(next_word_is(line, *index, context, r_eqv.data)){
			STRING_CONST(add_template, "\t%tmp# = xor i32 %tmp#, %tmp#\r\t%tmp# = xor i32 %tmp#, 4294967295\r");
			template = add_template.data;
			*index += LEN(r_eqv.data);
			skip_spaces(line, index);
			op = 1;
		}else if(next_word_is(line, *index, context, r_xor.data)){
			STRING_CONST(sub_template, "\t%tmp# = xor i32 %tmp#, %tmp#\r");
			template = sub_template.data;
			*index += LEN(r_xor.data);
			skip_spaces(line, index);
			op = 2;
		}else if(next_word_is(line, *index, context, r_imp.data)){
			STRING_CONST(sub_template, "\t%tmp# = xor i32 %tmp#, 4294967295\r\t%tmp# = or i32 %tmp#, %tmp#\r");
			template = sub_template.data;
			*index += LEN(r_imp.data);
			skip_spaces(line, index);
			op = 3;
		}else{
			break;
		}
		INTEGER left;
		INTEGER right;
		INTEGER using;
		struct expression_t right_expression;
		to_integer(context, expression, &left);
		if(! parse_or(line, index, context, &right_expression)){
			STRING_CONST(message, "Right term required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
		to_integer(context, &right_expression, &right);
		expression->result = ++ context->temp;
		expression->type = &integer_type;
		using = 0;
		if(op == 3){
			INTEGER x = ++ context->temp;
			using_double(&context->code, template, &using, x);
			using_double(&context->code, template, &using, left);
			left = x;
		}
		using_double(&context->code, template, &using, expression->result);
		using_double(&context->code, template, &using, left);
		using_double(&context->code, template, &using, right);
		if(op == 1){
			INTEGER x = expression->result;
			expression->result = ++ context->temp;
			using_double(&context->code, template, &using, expression->result);
			using_double(&context->code, template, &using, x);
		}
		using_rest(&context->code, template, &using);
		string_release(template);
	}
	return result;
}

LOGICAL parse_expression(STRING line, INTEGER *index, struct context_t *context, struct expression_t *expression, LOGICAL error)
{
	LOGICAL result;
	result = parse_eqv_xor_imp(line, index, context, expression);
	if(! result && error){
		STRING_CONST(message, "Expression required.");
		compile_time_error(context->line_number_in_file, message.data);
	}
	return result;
}

void to_logical(struct context_t *context, struct expression_t *expression, INTEGER *result)
{
	STRING_CONST(itol, "\t%tmp# = icmp ne i32 %tmp#, 0\r");
	STRING_CONST(ftol, "\t%tmp# = fcmp one float %tmp#, 0.0\r");
	STRING_CONST(dtol, "\t%tmp# = fcmp one double %tmp#, 0.0\r");
	
	INTEGER using;
	if(expression->type == &logical_type){
		*result = expression->result;
	}else if(expression->type == &integer_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, itol.data, &using, *result);
		using_double(&context->code, itol.data, &using, expression->result);
		using_rest(&context->code, itol.data, &using);
	}else if(expression->type == &single_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, ftol.data, &using, *result);
		using_double(&context->code, ftol.data, &using, expression->result);
		using_rest(&context->code, ftol.data, &using);
	}else if(expression->type == &double_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, dtol.data, &using, *result);
		using_double(&context->code, dtol.data, &using, expression->result);
		using_rest(&context->code, dtol.data, &using);
	}else{
		STRING_CONST(message, "Logical value required.");
		compile_time_error(context->line_number_in_file, message.data);
	}
}

void parse_logical_expression(STRING line, INTEGER *index, struct context_t *context, INTEGER *result)
{
	struct expression_t expression = {0, NULL};
	parse_expression(line, index, context, &expression, 1);
	to_logical(context, &expression, result);
}

void to_integer(struct context_t *context, struct expression_t *expression, INTEGER *result)
{
	STRING_CONST(ltoi, "\t%tmp# = select i1 %tmp#, i32 -1, i32 0\r");
	STRING_CONST(stoi, "\t%tmp# = fptosi float %tmp# to i32\r");
	STRING_CONST(dtoi, "\t%tmp# = fptosi double %tmp# to i32\r");
	
	INTEGER using;
	if(expression->type == &logical_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, ltoi.data, &using, *result);
		using_double(&context->code, ltoi.data, &using, expression->result);
		using_rest(&context->code, ltoi.data, &using);
	}else if(expression->type == &integer_type){
		*result = expression->result;
	}else if(expression->type == &single_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, stoi.data, &using, *result);
		using_double(&context->code, stoi.data, &using, expression->result);
		using_rest(&context->code, stoi.data, &using);
	}else if(expression->type == &double_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, dtoi.data, &using, *result);
		using_double(&context->code, dtoi.data, &using, expression->result);
		using_rest(&context->code, dtoi.data, &using);
	}else{
		STRING_CONST(message, "Integer required.");
		compile_time_error(context->line_number_in_file, message.data);
	}
}

void parse_integer_expression(STRING line, INTEGER *index, struct context_t *context, INTEGER *result)
{
	struct expression_t expression = {0, NULL};
	parse_expression(line, index, context, &expression, 1);
	to_integer(context, &expression, result);
}

void to_single(struct context_t *context, struct expression_t *expression, INTEGER *result)
{
	STRING_CONST(ltof, "\t%tmp# = select i1 %tmp#, float -1.0, float 0.0\r");
	STRING_CONST(itof, "\t%tmp# = sitofp i32 %tmp# to float\r");
	STRING_CONST(dtof, "\t%tmp# = fptrunc double %tmp# to float\r");
	
	INTEGER using;
	if(expression->type == &logical_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, ltof.data, &using, *result);
		using_double(&context->code, ltof.data, &using, expression->result);
		using_rest(&context->code, ltof.data, &using);
	}else if(expression->type == &integer_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, itof.data, &using, *result);
		using_double(&context->code, itof.data, &using, expression->result);
		using_rest(&context->code, itof.data, &using);
	}else if(expression->type == &single_type){
		*result = expression->result;
	}else if(expression->type == &double_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, dtof.data, &using, *result);
		using_double(&context->code, dtof.data, &using, expression->result);
		using_rest(&context->code, dtof.data, &using);
	}else{
		STRING_CONST(message, "Single floating-point value required.");
		compile_time_error(context->line_number_in_file, message.data);
	}
}

void parse_single_expression(STRING line, INTEGER *index, struct context_t *context, INTEGER *result)
{
	struct expression_t expression = {0, NULL};
	parse_expression(line, index, context, &expression, 1);
	to_single(context, &expression, result);
}

void to_double(struct context_t *context, struct expression_t *expression, INTEGER *result)
{
	STRING_CONST(ltod, "\t%tmp# = select i1 %tmp#, double -1.0, double 0.0\r");
	STRING_CONST(itod, "\t%tmp# = sitofp i32 %tmp# to double\r");
	STRING_CONST(stod, "\t%tmp# = fpext float %tmp# to double\r");
	
	INTEGER using;
	if(expression->type == &logical_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, ltod.data, &using, *result);
		using_double(&context->code, ltod.data, &using, expression->result);
		using_rest(&context->code, ltod.data, &using);
	}else if(expression->type == &integer_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, itod.data, &using, *result);
		using_double(&context->code, itod.data, &using, expression->result);
		using_rest(&context->code, itod.data, &using);
	}else if(expression->type == &single_type){
		*result = ++ context->temp;
		using = 0;
		using_double(&context->code, stod.data, &using, *result);
		using_double(&context->code, stod.data, &using, expression->result);
		using_rest(&context->code, stod.data, &using);
	}else if(expression->type == &double_type){
		*result = expression->result;
	}else{
		STRING_CONST(message, "Double floating-point value required.");
		compile_time_error(context->line_number_in_file, message.data);
	}
}

void parse_double_expression(STRING line, INTEGER *index, struct context_t *context, INTEGER *result)
{
	struct expression_t expression = {0, NULL};
	parse_expression(line, index, context, &expression, 1);
	to_double(context, &expression, result);
}

void parse_string_expression(STRING line, INTEGER *index, struct context_t *context, INTEGER *result)
{
	struct expression_t expression = {0, NULL};
	parse_expression(line, index, context, &expression, 1);
	if(expression.type == &string_type){
		*result = expression.result;
	}else{
		STRING_CONST(message, "String required.");
		compile_time_error(context->line_number_in_file, message.data);
	}
}

void parse_symbol(STRING line, INTEGER *index, struct context_t *context, CHARACTER c)
{
	if(line[*index] != c){
		STRING_CONST(message, "\"#\" required.");
		STRING s = message.data;
		string_addref(s);
		unique_string(&s);
		((char *)s)[1] = c;
		compile_time_error(context->line_number_in_file, s);
	}
	++ *index;
	skip_spaces(line, index);
}

void store_variable(struct context_t *context, struct user_defined_t *variable, struct expression_t *expression)
{
	STRING_CONST(iassign, "\tstore i32 %tmp#, i32* #\r");
	STRING_CONST(fassign, "\tstore float %tmp#, float* #\r");
	STRING_CONST(dassign, "\tstore double %tmp#, double* #\r");
	STRING_CONST(sassign, 
		"\t%tmp# = load i8** #\r"
		"\tcall void @string_release(i8* %tmp#)\r"
		"\tstore i8* %tmp#, i8** #\r"
		"\tcall void @string_addref(i8* %tmp#)\r");

	STRING name = NULL;
	INTEGER value;
	INTEGER using;
	name = to_internal_name(variable->name);
	if(variable->type == &integer_type){
		to_integer(context, expression, &value);
		using = 0;
		using_double(&context->code, iassign.data, &using, value);
		using_string(&context->code, iassign.data, &using, name);
		using_rest  (&context->code, iassign.data, &using);
	}else if(variable->type == &single_type){
		to_single(context, expression, &value);
		using = 0;
		using_double(&context->code, fassign.data, &using, value);
		using_string(&context->code, fassign.data, &using, name);
		using_rest  (&context->code, fassign.data, &using);
	}else if(variable->type == &double_type){
		to_double(context, expression, &value);
		using = 0;
		using_double(&context->code, dassign.data, &using, value);
		using_string(&context->code, dassign.data, &using, name);
		using_rest  (&context->code, dassign.data, &using);
	}else{
		INTEGER t = ++ context->temp;
		using = 0;
		using_double(&context->code, sassign.data, &using, t);
		using_string(&context->code, sassign.data, &using, name);
		using_double(&context->code, sassign.data, &using, t);
		using_double(&context->code, sassign.data, &using, expression->result);
		using_string(&context->code, sassign.data, &using, name);
		using_double(&context->code, sassign.data, &using, expression->result);
		using_rest  (&context->code, sassign.data, &using);
	}
	string_release(name);
}

LOGICAL parse_statement(STRING line, INTEGER *index, struct context_t *context, struct user_defined_t **assignment)
{
	STRING statement = NULL;
	STRING temp = NULL;
	INTEGER start;
	INTEGER found;
	INTEGER i;
	struct type_t const *type;
	start = *index;
	*index = take_word(line, start);
	if(*index == start){
		if(line[*index] == '?'){
			++ *index;
			string_release(statement);
			statement = r_print.data;
		}else{
			STRING_CONST(message, "Any word required.");
			compile_time_error(context->line_number_in_file, message.data);
		}
	}else{
		string_release(temp);
		temp = MID(line, start + 1, *index - start);
		string_release(statement);
		statement = UCASE(temp);
	}
	*assignment = query_user_defined(context->user_defined_list, statement);
	found = -1;
	if(*assignment == NULL){
		for(i = 0; i < statement_count; ++i){
			if(string_compare(statements[i].keyword, statement) == 0){
				found = i;
				break;
			}
		}
	}
	if(found < 0){
		struct expression_t expression = {0, NULL};
		/* assignment */
		if(*assignment == NULL){
			parse_type_postfix(line, index, context, statement, &type);
			*assignment = get_user_defined(&context->user_defined_list, statement, type);
		}
		parse_symbol(line, index, context, '=');
		parse_expression(line, index, context, &expression, 1);
		store_variable(context, *assignment, &expression);
		cleanup(context);
	}else{
		/* internal statement */
		*index = start;
		if(! statements[found].func(line, index, context)){
			STRING_CONST(message, "Syntax error.");
			compile_time_error(context->line_number_in_file, message.data);
		}
	}
	string_release(statement);
	string_release(temp);
	return 1;
}

LOGICAL parse_multi_statement(STRING line, INTEGER *index, struct context_t *context, LOGICAL multi_statement)
{
	LOGICAL had_statement = multi_statement;
	struct user_defined_t *user_defined;
	while(*index < LEN(line)){
		if(line[*index] == ':'){
			++ *index;
			skip_spaces(line, index);
			had_statement = 0;
		}
		if(had_statement){
			STRING_CONST(message, "\":\" required.");
			if(next_word_is(line, *index, context, r_else.data)){
				break;
			}
			compile_time_error(context->line_number_in_file, message.data);
		}
		parse_statement(line, index, context, &user_defined);
		had_statement = 1;
	}
	return 1;
}

LOGICAL parse(INTEGER source, struct context_t *context)
{
	STRING_CONST(label_format, "\tbr label %L#\rL#:\r");
	LOGICAL result = 1;
	STRING line = NULL;
	STRING label = NULL;
	STRING temp = NULL;
	STRING statement = NULL;
	struct label_t *label_item;
	INTEGER line_number;
	INTEGER start;
	INTEGER index;
	INTEGER using;
	INTEGER had_statement;
	while(! EOF(source)){
		line_input(source, &line);
		++ context->line_number_in_file;
		if(line == NULL){
			line = STRPTR(line);
		}
		index = 0;
		skip_spaces(line, &index);
		
		/* take line number */
		start = index;
		index = take_line_number(line, index);
		if(index > start){
			string_release(temp);
			temp = LEFT(line, index);
			line_number = VAL(temp);
			if(line_number == 0){
				STRING_CONST(message, "Invalid line number.");
				compile_time_error(context->line_number_in_file, message.data);
			}
			label_item = get_label(&context->label_list, temp, line_number);
			if(label_item->state == defined){
				STRING_CONST(message, "Duplicative line number.");
				compile_time_error(context->line_number_in_file, message.data);
			}
			label_item->state = defined;
			label_item->data_position = context->data_count;
			using = 0;
			using_string(&context->code, label_format.data, &using, temp);
			using_string(&context->code, label_format.data, &using, temp);
			using_rest(&context->code, label_format.data, &using);
			skip_spaces(line, &index);
		}else{
			line_number = 0;
		}
		/* take label */
		if(line[index] == '*'){
			++ index;
			start = index;
			index = take_word(line, start);
			if(index == start){
				STRING_CONST(message, "Invalid label.");
				compile_time_error(context->line_number_in_file, message.data);
			}
			string_release(temp);
			temp = MID(line, start + 1, index - start);
			string_release(label);
			label = UCASE(temp);
			label_item = get_label(&context->label_list, label, 0);
			if(label_item->state == defined){
				STRING_CONST(message, "Duplicative label.");
				compile_time_error(context->line_number_in_file, message.data);
			}
			label_item->state = defined;
			label_item->data_position = context->data_count;
			using = 0;
			using_string(&context->code, label_format.data, &using, label);
			using_string(&context->code, label_format.data, &using, label);
			using_rest(&context->code, label_format.data, &using);
			skip_spaces(line, &index);
			had_statement = 1;
		}else{
			string_release(label);
			label = NULL;
			had_statement = 0;
		}
		/* statements */
		parse_multi_statement(line, &index, context, had_statement);
	}
	/* check */
	for(label_item = context->label_list; label_item != NULL; label_item = label_item->next){
		if(label_item->state == referenced){
			STRING_CONST(message, "Undefined label or line number.");
			compile_time_error(label_item->line_number_in_file, message.data);
		}
	}
	string_release(line);
	string_release(label);
	string_release(temp);
	string_release(statement);
	return result;
}

void cleanup(struct context_t *context)
{
	STRING_CONST(strrel, "\tcall void @string_release(i8* %tmp#)\r");

	INTEGER using;
	struct temp_in_expression_t *list = context->temp_list;
	while(list != NULL){
		struct temp_in_expression_t *next = list->next;
		if(list->type == &string_type){
			using = 0;
			using_double(&context->code, strrel.data, &using, list->temp);
			using_rest(&context->code, strrel.data, &using);
		}
		deallocate(list);
		list = next;
	}
	context->temp_list = NULL;
}

void make_list(INTEGER source, INTEGER output)
{
	STRING_CONST(close, "\\00\"}\r");
	STRING_CONST(escaped_backslash, "\\5c");
	STRING_CONST(escaped_doublequote, "\\22");
	STRING_CONST(new_line_marker, "\\0d");
	STRING line = NULL;
	STRING list = NULL;
	STRING temp = NULL;
	INTEGER length = 0;
	INTEGER i;
	INTEGER index;
	while(! EOF(source)){
		line_input(source, &line);
		length += LEN(line) + 1;
		for(i = 0; i < LEN(line); ++i){
			if(line[i] == '\"'){
				string_append(&list, escaped_doublequote.data);
			}else if(line[i] == '\\'){
				string_append(&list, escaped_backslash.data);
			}else{
				string_release(temp);
				temp = MID(line, i + 1, 1);
				string_append(&list, temp);
			}
		}
		string_append(&list, new_line_marker.data);
	}
	index = 0;
	PRINT_USING(output, 
		"%list_t = type {i32, i32, [# x i8]}\r"
		"@list = internal constant %list_t {i32 -1, i32 #, [# x i8] c\"",
		PUD(length + 1) /* including \0 */
		PUD(length)
		PUD(length + 1)); /* including \0 */
	print_string(output, list);
	print_string(output, close.data);
	string_release(list);
	string_release(line);
	string_release(temp);
}

void make_string_literal(INTEGER output, struct string_literal_t *list)
{
	struct string_literal_t *i;
	for(i = list; i != NULL; i = i->next){
		PRINT_USING(output,
			"%string#_t = type {i32, i32, [# x i8]}\r"
			"@string# = internal constant %string#_t {i32 -1, i32 #, [# x i8] c\"@\\00\"}\r",
			PUD(i->number)
			PUD(LEN(i->value) + 1)
			PUD(i->number)
			PUD(i->number)
			PUD(LEN(i->value))
			PUD(LEN(i->value) + 1)
			PUS(i->value));
	}
}

void make_jump_table(INTEGER output, struct label_t *list)
{
	STRING_CONST(switch_h, "\tswitch i32 %start, label %L0 [");
	STRING_CONST(switch_i, "\r\ti32 #, label %L#");
	STRING_CONST(switch_f, "]\rL0:\r");

	struct label_t *i;
	INTEGER using;
	print_string(output, switch_h.data);
	for(i = list; i != NULL; i = i->next){
		using = 0;
		print_using_double(output, switch_i.data, &using, i->number);
		print_using_string(output, switch_i.data, &using, i->label);
		print_using_rest(output, switch_i.data, &using);
	}
	print_string(output, switch_f.data);
}

void make_variables(INTEGER output, struct user_defined_t *list)
{
	STRING_CONST(idef, " = global i32 0\r");
	STRING_CONST(sdef, " = global float 0.0\r");
	STRING_CONST(ddef, " = global double 0.0\r");
	struct user_defined_t *i;
	for(i = list; i != NULL; i = i->next){
		STRING s = to_internal_name(i->name);
		print_string(output, s);
		if(i->type == &integer_type){
			print_string(output, idef.data);
		}else if(i->type == &single_type){
			print_string(output, sdef.data);
		}else if(i->type == &double_type){
			print_string(output, ddef.data);
		}else if(i->type->kind == tk_array){
			if(i->type->element == &character_type){
				print_string(output, S(" = global i8* null\r").data);
			}else{
				print_string(output, S(" = global float* null\r").data);
			}
		}else{
			print_string(output, S(" = global i8* null; internal error\r").data);
		}
		string_release(s);
	}
}

void make_data(INTEGER output, STRING *data, INTEGER data_count, struct label_t *list)
{
	STRING_CONST(c, 
		"%data_#_t = type {i32, i32, [# x i8]}\r"
		"@data_# = internal constant %data_#_t {i32 -1, i32 #, [# x i8] c\"@\\00\"}\r");

	STRING_CONST(h, "@data = internal constant [# x i8*] [\r");
	STRING_CONST(e, "\ti8* getelementptr(%data_#_t* @data_#, i32 0, i32 2, i32 0),\r");
	STRING_CONST(f, "\ti8* null]\r@data_reading = internal global i8** getelementptr([# x i8*]* @data, i32 0, i32 0)\r");

	STRING_CONST(rh, "define void @restore(i32 %l){\r\tswitch i32 %l, label %L0 [");
	STRING_CONST(ri, "\r\ti32 #, label %L#");
	STRING_CONST(rj, "]\r");
	STRING_CONST(rk, "L#:\r\tstore i8** getelementptr([# x i8*]* @data, i32 0, i32 #), i8*** @data_reading\r\tret void\r");
	STRING_CONST(rf, "}\r");

	INTEGER i;
	INTEGER using;
	struct label_t *li;
	for(i = 0; i < data_count; ++ i){
		using = 0;
		print_using_double(output, c.data, &using, i);
		print_using_double(output, c.data, &using, LEN(data[i]) + 1);
		print_using_double(output, c.data, &using, i);
		print_using_double(output, c.data, &using, i);
		print_using_double(output, c.data, &using, LEN(data[i]));
		print_using_double(output, c.data, &using, LEN(data[i]) + 1);
		print_using_string(output, c.data, &using, data[i]);
		print_using_rest  (output, c.data, &using);
	}
	using = 0;
	print_using_double(output, h.data, &using, data_count + 1);
	print_using_rest  (output, h.data, &using);
	for(i = 0; i < data_count; ++ i){
		using = 0;
		print_using_double(output, e.data, &using, i);
		print_using_double(output, e.data, &using, i);
		print_using_rest  (output, e.data, &using);
	}
	using = 0;
	print_using_double(output, f.data, &using, data_count + 1);
	print_using_rest  (output, f.data, &using);
	print_string(output, rh.data);
	for(li = list; li != NULL; li = li->next){
		using = 0;
		print_using_double(output, ri.data, &using, li->number);
		print_using_string(output, ri.data, &using, li->label);
		print_using_rest  (output, ri.data, &using);
	}
	print_string(output, rj.data);
	using = 0;
	print_using_double(output, rk.data, &using, 0);
	print_using_double(output, rk.data, &using, data_count + 1);
	print_using_double(output, rk.data, &using, 0);
	print_using_rest  (output, rk.data, &using);
	for(li = list; li != NULL; li = li->next){
		using = 0;
		print_using_string(output, rk.data, &using, li->label);
		print_using_double(output, rk.data, &using, data_count + 1);
		print_using_double(output, rk.data, &using, li->data_position);
		print_using_rest  (output, rk.data, &using);
	}
	print_string(output, rf.data);
}

INTEGER compile(STRING source, STRING output)
{
	INTEGER result;
	INTEGER sf;
	INTEGER of;
	INTEGER i;
	struct context_t context;
	context.code = NULL;
	context.label_list = NULL;
	context.user_defined_list = NULL;
	context.string_literal_list = NULL;
	context.temp_list = NULL;
	context.nest_list = NULL;
	context.line_number_in_file = 0;
	context.temp = 0;
	context.level = 0;
	context.data = NULL;
	context.data_count = 0;
	context.data_used = 0;
	context.list_used = 0;
	context.jump_table_used = 0;
	context.type_list = &double_type;
	for(i = 0; i <= 'Z' - 'A'; ++ i){
		context.default_type[i] = &single_type;
	}
	/* parse */
	sf = FREEFILE();
	OPEN(source, FOR_INPUT, sf);
	result = parse(sf, &context);
	CLOSE(sf);
	/* generate code */
	if(result){
		of = FREEFILE();
		OPEN(output, FOR_OUTPUT, of);
		print_string(of, main_template.data);
		if(context.list_used){
			sf = FREEFILE();
			OPEN(source, FOR_INPUT, sf);
			make_list(sf, of);
			CLOSE(sf);
		}
		make_variables(of, context.user_defined_list);
		make_string_literal(of, context.string_literal_list);
		if(context.data_used){
			make_data(of, context.data, context.data_count, context.label_list);
		}
		print_string(of, S("define void @program(i32 %start){\r").data);
		if(context.jump_table_used){
			make_jump_table(of, context.label_list);
		}
		print_string(of, context.code);
		print_string(of, S("\ttail call void @END() noreturn\r\tunreachable\r}\r").data);
		CLOSE(of);
	}
	string_release(context.code);
	free_label_list(context.label_list);
	free_type_list(context.type_list);
	free_user_defined_list(context.user_defined_list);
	free_string_literal_list(context.string_literal_list);
	free_temp_list(context.temp_list);
	free_nest_list(context.nest_list);
	for(i = 0; i < context.data_count; ++ i){
		string_release(context.data[i]);
	}
	deallocate(context.data);
	return result;
}

void program(void)
{
	enum compiler_action_t { to_ll, to_bc, to_linked_bc, to_s, to_exe, to_run };
	STRING arg = NULL;
	STRING source = NULL;
	STRING base = NULL;
	STRING output = NULL;
	STRING asm_file = NULL;
	STRING bc_file = NULL;
	STRING s_file = NULL;
	STRING exe_file = NULL;
	STRING command = NULL;
	STRING temp = NULL;
	INTEGER i;
	INTEGER version = 0;
	enum compiler_action_t action = to_linked_bc;
	for(i = 1; i <= IARGC(); ++i){
		GETARG(i, &arg);
		if(string_compare(arg, S("--version").data) == 0
			|| string_compare(arg, S("--help").data) == 0)
		{
			version = 1;
		}else if(string_compare(arg, S("-o").data) == 0){
			++ i;
			GETARG(i, &arg);
			output = arg;
			string_addref(output);
		}else if(string_compare(arg, S("-S").data) == 0){
			action = to_ll;
		}else if(string_compare(arg, S("-c").data) == 0){
			action = to_bc;
		}else if(string_compare(arg, S("-x").data) == 0){
			action = to_exe;
		}else if(string_compare(arg, S("-r").data) == 0){
			action = to_run;
		}else if(arg[0] == '-'){
			print_string(standard_error, S("Invalid option.\r").data);
			version = 1;
		}else{
			source = arg;
			string_addref(source);
		}
	}
	if(LEN(source) == 0 || version){
		usage();
		basic_exit_code = 1;
	}else{
		base = RIGHT(source, 4);
		if(string_compare(base, S(".bas").data) == 0){
			string_release(base);
			base = LEFT(source, LEN(source) - 4);
		}else{
			string_release(base);
			base = source;
			string_addref(base);
		}
		if(action <= to_ll){
			if(LEN(output) == 0){
				output = string_concatenate(base, S(".ll").data);
			}
			asm_file = output;
			string_addref(asm_file);
		}else{
			GETENV(S("TMP").data, &temp);
			if(LEN(temp) > 0 && temp[LEN(temp) - 1] != dir_sep.data[0]){
				string_append(&temp, dir_sep.data);
			}
			asm_file = string_concatenate(temp, S("temp.ll").data);
			if(action >= to_exe){
				bc_file = string_concatenate(temp, S("temp.bc").data);
				s_file = string_concatenate(temp, S("temp.s").data);
				if(LEN(output) == 0){
					if(action >= to_run){
						exe_file = string_concatenate(temp, S("temp.exe").data);
					}else{
						exe_file = string_concatenate(base, S(".exe").data);
					}
				}else{
					exe_file = output;
					string_addref(exe_file);
				}
			}else{
				if(LEN(output) == 0){
					bc_file = string_concatenate(base, S(".bc").data);
				}else{
					bc_file = output;
					string_addref(bc_file);
				}
			}
		}
		basic_exit_code = ! compile(source, asm_file);
		if(basic_exit_code == 0 && action >= to_bc){
			/* .ll -> .bc */
			command = string_concatenate(S("llvm-as -f -o ").data, bc_file);
			string_append(&command, S(" ").data);
			string_append(&command, asm_file);
			if(SHELL(command) != 0){
				basic_exit_code = 1;
			}else{
				KILL(asm_file);
				if(action >= to_linked_bc){
					/* merge libbasicll.bc to .bc */
					string_release(command);
					command = string_concatenate(S("llvm-link -f -o ").data, bc_file);
					string_append(&command, S(" ").data);
					string_append(&command, bc_file);
					string_append(&command, S(" libbasicll.bc").data);
					if(SHELL(command) != 0){
						basic_exit_code = 1;
					}else if(action >= to_exe){
						/* .bc -> .s */
						string_release(command);
						command = string_concatenate(S("llc -f -o ").data, s_file);
						string_append(&command, S(" ").data);
						string_append(&command, bc_file);
						if(SHELL(command) != 0){
							basic_exit_code = 1;
						}else{
							KILL(bc_file);
							/* .s -> .exe */
							string_release(command);
							command = string_concatenate(S("gcc -o ").data, exe_file);
							string_append(&command, S(" ").data);
							string_append(&command, s_file);
							if(SHELL(command) != 0){
								basic_exit_code = 1;
							}else{
								KILL(s_file);
								if(action >= to_run){
									basic_exit_code = SHELL(exe_file);
									if(LEN(output) == 0){
										KILL(exe_file);
									}
								}
							}
						}
					}
				}
			}
		}
	}
	string_release(arg);
	string_release(source);
	string_release(base);
	string_release(output);
	string_release(asm_file);
	string_release(bc_file);
	string_release(s_file);
	string_release(exe_file);
	string_release(command);
	string_release(temp);
}

/* entry point */

int main(int argc, char **args)
{
	int result;
	basic_argc = argc;
	basic_args = args;
	if(setjmp(basic_on_exit) == 0){
		program();
		result = basic_exit_code;
	}else{
		result = ERR();
		if(result != 0){
			print_string(standard_error, S("Error!").data);
			print_double(standard_error, result);
			print_line(standard_error);
			result += 100;
		}else{
			result = basic_exit_code;
		}
	}
	return result;
}
